subroutine ApplyCorrections

use CommonVars, only: Prec,Corrections,NumNodesTotal,GridIndMat, &
   Pres,Temp,Sali,TempNodeTypes,OldTemp,NumNodesX,NumNodesZ,     &
   PresNodeTypes,SaliNodeTypes,OldSali,MaxPres,MinPres,MaxTemp,  &
   MinTemp,MaxSali,MinSali,PresBoundVals,TempBoundVals,          &
   SaliBoundVals,PrevCorrect,LogFile
implicit none
real(selected_real_kind(Prec)) :: InterfHeatFlux2,HeatFlux,SaltFlux, &
   InterfSaltFlux2,NewPres,NewTemp
integer :: L,I,J,K,NodeNumber

do L=1,2*NumNodesTotal
   PrevCorrect(L) = Corrections(L)
   if (L <= NumNodesTotal) then
      I = GridIndMat(L,1)
      J = GridIndMat(L,2)
      K = GridIndMat(L,3)
   elseif (L <= 2*NumNodesTotal) then
      I = GridIndMat(L-NumNodesTotal,1)
      J = GridIndMat(L-NumNodesTotal,2)
      K = GridIndMat(L-NumNodesTotal,3)
   end if

   if (L <= NumNodesTotal) then
      if (PresNodeTypes(I,J,K).ne.'C') then
         NewPres = Pres(I,J,K) + Corrections(L)
         if ((NewPres>MinPres).and.(NewPres<MaxPres)) then
            Pres(I,J,K) = NewPres
         end if
      else
         Pres(I,J,K) = PresBoundVals(I,J,K)
      end if
   elseif (L <= 2*NumNodesTotal) then
      if ((TempNodeTypes(I,J,K).ne.'C').and. &
          (TempNodeTypes(I,J,K).ne.'U')) then
         NewTemp = Temp(I,J,K) + Corrections(L)
         if ((NewTemp>MinTemp).and.(NewTemp<MaxTemp)) then
            Temp(I,J,K) = NewTemp
         end if
      elseif (TempNodeTypes(I,J,K)=='C') then
         Temp(I,J,K) = TempBoundVals(I,J,K)
      end if
   end if
end do

! enforce any flow-through boundary conditions
! note that only top and bottom boundaries can currently be set
! to flow-through conditions
do L=1,NumNodesTotal
   I = GridIndMat(L,1)
   J = GridIndMat(L,2)
   K = GridIndMat(L,3)
   if (TempNodeTypes(I,J,K)=='U') then
      if (K==1) then
         HeatFlux = InterfHeatFlux2(L,'B','L',Temp(I,J,K),Pres(I,J,K), &
                    Sali(I,J,K)) + &
                    InterfHeatFlux2(L,'B','V',Temp(I,J,K),Pres(I,J,K), &
                    Sali(I,J,K))
         SaltFlux = InterfSaltFlux2(L,'B','L',Temp(I,J,K),Pres(I,J,K), &
                    Sali(I,J,K)) + &
                    InterfSaltFlux2(L,'B','V',Temp(I,J,K),Pres(I,J,K), &
                    Sali(I,J,K))
      else if (K==NumNodesZ) then
         HeatFlux = InterfHeatFlux2(L,'T','L',Temp(I,J,K),Pres(I,J,K), &
                    Sali(I,J,K)) + &
                    InterfHeatFlux2(L,'T','V',Temp(I,J,K),Pres(I,J,K), &
                    Sali(I,J,K))
         SaltFlux = InterfSaltFlux2(L,'T','L',Temp(I,J,K),Pres(I,J,K), &
                    Sali(I,J,K)) + &
                    InterfSaltFlux2(L,'T','V',Temp(I,J,K),Pres(I,J,K), &
                    Sali(I,J,K))
      end if
      if (K==1) then
         if (HeatFlux > 0.D0) then
            Temp(I,J,K) = TempBoundVals(I,J,K)
         else
            Temp(I,J,K) = Temp(I,J,K+1)
         end if
         if (SaltFlux > 0.D0) then
            Sali(I,J,K) = SaliBoundVals(I,J,K)
         else
            Sali(I,J,K) = Sali(I,J,K+1)
         end if
      elseif (K==NumNodesZ) then
         if (HeatFlux < 0.D0) then
            Temp(I,J,K) = TempBoundVals(I,J,K)
         else
            Temp(I,J,K) = Temp(I,J,K-1)
         end if
         if (SaltFlux < 0.D0) then
            Sali(I,J,K) = SaliBoundVals(I,J,K)
         else
            Sali(I,J,K) = Sali(I,J,K-1)
         end if
      end if
   end if
end do
end
