subroutine CalcInterface
! Purpose: To calculate the values of various parameters at the     
!          node interfaces.

! Note: if the nodes are arranged, for example, like this
! |  N  |  N  |  N  |  N  |
! then the number of nodes is 4, while the number of interfaces
! between nodes is 3.  The number of interfaces in a given direction
! is always 1 less than the number of nodes in that direction. Also,
! the nodes are numbered increasing from west to east and increasing
! from top to bottom, as are the node interfaces.

use CommonVars, only: Prec,NumNodesX,NumNodesY,NumNodesZ,Porosities, &
   XInterfPoros,YInterfPoros,ZInterfPoros,InterfPermsX,              &
   XInterfLiqVolSats,InterfPermsY,YInterfLiqVolSats,InterfPermsZ,    &
   ZInterfLiqVolSats,RockPerms,LiqVolSats,XInterfVapDens,            &
   XInterfLiqDens,LiquidDens,VaporDens,ZInterfVapDens,               &
   ZInterfLiqDens,XInterfVapSali,XInterfLiqSali,ZInterfVapSali,      &
   ZInterfLiqSali,LiquidSali,VaporSali,XInterfPres,ZInterfPres,Pres, &
   XInterfVapEnth,ZInterfVapEnth,XInterfLiqEnth,ZInterfLiqEnth,      &
   VaporEnth,LiquidEnth,LiqVelocX,LiqVelocY,LiqVelocZ,VapVelocX,     &
   VapVelocY,VapVelocZ,RockThermCond,ZNodeWidths,ZInterfTemp,        &
   VapVolSats,XInterfTemp,XNodeWidths,XInterfVapVolSats,VapVolSats,  &
   ZInterfVapVolSats,HaliteSat,Temp,XInterfLiqCp,ZInterfLiqCp,       &
   XInterfVapCp,ZInterfVapCp,XInterfLiqDens0,ZInterfLiqDens0,        &
   XInterfVapDens0,ZInterfVapDens0,Grav,XInterfLiqVisc,              &
   ZInterfLiqVisc,XInterfVapVisc,ZInterfVapVisc,LiquidVisc,          &
   VaporVisc,XInterfEffCond,ZInterfEffCond,FluidThermCond,           &
   SaltChemDiff,XInterfLiqDiff,XInterfVapDiff,ZInterfLiqDiff,        &
   ZInterfVapDiff
        
implicit none
real (selected_real_kind(Prec)) :: AvgPoros,Poros1,Poros2,           &
   HarmonicMean,AvgPerm,Perm1,Perm2,Pres1,Pres2,AvgPres,             &
   RelInterfPos,NodeDist,Temp1,Temp2,AvgTemp,LiqSat1,LiqSat2,        &
   AvgLiqSat,VapSat1,VapSat2,AvgVapSat,Dens1,Dens2,AvgDens,          &
   LiqDirection,VapDirection,PresGrad,FViscosity,SaltLiqDiff,        &
   SaltVapDiff
integer :: I,J,K
! -------------------------------------------------------------------
!                       X Direction Interfaces
! -------------------------------------------------------------------
if ((NumNodesX-1).ne.0) then
   do I=1,NumNodesX-1
      do J=1,NumNodesY
         do K=1,NumNodesZ
            NodeDist = XNodeWidths(I)/2.D0 + &
                       XNodeWidths(I+1)/2.D0
            RelInterfPos = (XNodeWidths(I)/2.D0)/NodeDist
            !---------------------------------------
            Poros1 = Porosities(I,J,K)*(1.D0-HaliteSat(I,J,K))
            Poros2 = Porosities(I+1,J,K)*(1.D0-HaliteSat(I+1,J,K))
            AvgPoros = HarmonicMean(Poros1,Poros2,RelInterfPos)
            XInterfPoros(I,J,K) = AvgPoros
            !---------------------------------------
            Perm1 = RockPerms(I,J,K)
            Perm2 = RockPerms(I+1,J,K)
            AvgPerm = HarmonicMean(Perm1,Perm2,RelInterfPos)
            InterfPermsX(I,J,K) = AvgPerm
            !---------------------------------------
            XInterfEffCond(I,J,K) = &
               XInterfPoros(I,J,K)*FluidThermCond + &
               (1.D0-XInterfPoros(I,J,K))*RockThermCond
            !---------------------------------------
            Pres1 = Pres(I,J,K)
            Pres2 = Pres(I+1,J,K)
            AvgPres = (1.D0-RelInterfPos)*Pres1+ &
                      RelInterfPos*Pres2
            XInterfPres(I,J,K) = AvgPres
            !---------------------------------------
            Temp1 = Temp(I,J,K)
            Temp2 = Temp(I+1,J,K)
            AvgTemp = (1.D0-RelInterfPos)*Temp1+ &
                      RelInterfPos*Temp2
            XInterfTemp(I,J,K) = AvgTemp
            !---------------------------------------
            Dens1 = LiquidDens(I,J,K)
            Dens2 = LiquidDens(I+1,J,K)
            AvgDens = (1.D0-RelInterfPos)*Dens1+ &
                      RelInterfPos*Dens2
            XInterfLiqDens0(I,J,K) = AvgDens
            !---------------------------------------
            Dens1 = VaporDens(I,J,K)
            Dens2 = VaporDens(I+1,J,K)
            AvgDens = (1.D0-RelInterfPos)*Dens1+ &
                      RelInterfPos*Dens2
            XInterfVapDens0(I,J,K) = AvgDens
            !---------------------------------------
            PresGrad = (Pres(I+1,J,K)-Pres(I,J,K))/ &
                       NodeDist
            LiqDirection = -PresGrad 
            VapDirection = -PresGrad
            !---------------------------------------
            ! Upstream weighting section
            !---------------------------------------
            if (LiqDirection<0.D0) then
               XInterfLiqDens(I,J,K)=LiquidDens(I+1,J,K)
               XInterfLiqSali(I,J,K)=LiquidSali(I+1,J,K)
               XInterfLiqEnth(I,J,K)=LiquidEnth(I+1,J,K)
               XInterfLiqVisc(I,J,K)=LiquidVisc(I+1,J,K)
               XInterfLiqCp(I,J,K)=LiquidEnth(I+1,J,K)/Temp(I+1,J,K)
               XInterfLiqVolSats(I,J,K)=LiqVolSats(I+1,J,K)
            elseif (LiqDirection>=0.D0) then
               XInterfLiqDens(I,J,K)=LiquidDens(I,J,K)
               XInterfLiqSali(I,J,K)=LiquidSali(I,J,K)
               XInterfLiqEnth(I,J,K)=LiquidEnth(I,J,K)
               XInterfLiqVisc(I,J,K)=LiquidVisc(I,J,K)
               XInterfLiqCp(I,J,K)=LiquidEnth(I,J,K)/Temp(I,J,K)
               XInterfLiqVolSats(I,J,K)=LiqVolSats(I,J,K)
            end if
            !---------------------------------------
            if (VapDirection<0.D0) then
               XInterfVapDens(I,J,K)=VaporDens(I+1,J,K)
               XInterfVapSali(I,J,K)=VaporSali(I+1,J,K)
               XInterfVapEnth(I,J,K)=VaporEnth(I+1,J,K)
               XInterfVapVisc(I,J,K)=VaporVisc(I+1,J,K)
               XInterfVapCp(I,J,K)=VaporEnth(I+1,J,K)/Temp(I+1,J,K)
               XInterfVapVolSats(I,J,K)=VapVolSats(I+1,J,K)
            elseif (VapDirection>=0.D0) then
               XInterfVapDens(I,J,K)=VaporDens(I,J,K)
               XInterfVapSali(I,J,K)=VaporSali(I,J,K)
               XInterfVapEnth(I,J,K)=VaporEnth(I,J,K)
               XInterfVapVisc(I,J,K)=VaporVisc(I,J,K)
               XInterfVapCp(I,J,K)=VaporEnth(I,J,K)/Temp(I,J,K)
               XInterfVapVolSats(I,J,K)=VapVolSats(I,J,K)
            end if
            !---------------------------------------
            ! these terms are needed for the diffusion part
            ! of the salt balance equation
            !---------------------------------------
            SaltLiqDiff = SaltChemDiff
            SaltVapDiff = 1.D4*SaltChemDiff
            XInterfLiqDiff(I,J,K) = SaltLiqDiff*XInterfPoros(I,J,K)* &
               XInterfLiqVolSats(I,J,K)
            XInterfVapDiff(I,J,K) = SaltVapDiff*XInterfPoros(I,J,K)* &
               XInterfVapVolSats(I,J,K)
            !---------------------------------------
         end do
      end do
   end do
end if
! -------------------------------------------------------------------
!                       Y Direction Interfaces
! -------------------------------------------------------------------
! Add if ever want the 3rd dimension
! -------------------------------------------------------------------
!                       Z Direction Interfaces
! -------------------------------------------------------------------
if ((NumNodesZ-1).ne.0) then
   do K=1,NumNodesZ-1
      do I=1,NumNodesX
         do J=1,NumNodesY
            NodeDist = ZNodeWidths(K)/2.D0 + &
                       ZNodeWidths(K+1)/2.D0
            RelInterfPos = (ZNodeWidths(K)/2.D0)/NodeDist
            !---------------------------------------
            Poros1 = Porosities(I,J,K)*(1.D0-HaliteSat(I,J,K))
            Poros2 = Porosities(I,J,K+1)*(1.D0-HaliteSat(I,J,K+1))
            AvgPoros = HarmonicMean(Poros1,Poros2,RelInterfPos)
            ZInterfPoros(I,J,K) = AvgPoros
            !---------------------------------------
            Perm1 = RockPerms(I,J,K)
            Perm2 = RockPerms(I,J,K+1)
            AvgPerm = HarmonicMean(Perm1,Perm2,RelInterfPos)
            InterfPermsZ(I,J,K) = AvgPerm
            !---------------------------------------
            ZInterfEffCond(I,J,K) = &
               ZInterfPoros(I,J,K)*FluidThermCond + &
               (1.D0-ZInterfPoros(I,J,K))*RockThermCond
            !---------------------------------------
            Pres1 = Pres(I,J,K)
            Pres2 = Pres(I,J,K+1)
            AvgPres = (1.D0-RelInterfPos)*Pres1+ &
                      RelInterfPos*Pres2
            ZInterfPres(I,J,K) = AvgPres
            !---------------------------------------
            Temp1 = Temp(I,J,K)
            Temp2 = Temp(I,J,K+1)
            AvgTemp = (1.D0-RelInterfPos)*Temp1+ &
                      RelInterfPos*Temp2
            ZInterfTemp(I,J,K) = AvgTemp
            !---------------------------------------
            Dens1 = LiquidDens(I,J,K)
            Dens2 = LiquidDens(I,J,K+1)
            AvgDens = (1.D0-RelInterfPos)*Dens1+ &
                      RelInterfPos*Dens2
            ZInterfLiqDens0(I,J,K) = AvgDens
            !---------------------------------------
            Dens1 = VaporDens(I,J,K)
            Dens2 = VaporDens(I,J,K+1)
            AvgDens = (1.D0-RelInterfPos)*Dens1+ &
                      RelInterfPos*Dens2
            ZInterfVapDens0(I,J,K) = AvgDens
            !---------------------------------------
            PresGrad = (Pres(I,J,K+1)-Pres(I,J,K))/ &
                       NodeDist
            LiqDirection = -(PresGrad - &
                             ZInterfLiqDens0(I,J,K)*Grav)
            VapDirection = -(PresGrad - &
                             ZInterfVapDens0(I,J,K)*Grav)
            !---------------------------------------
            ! Upstream weighting section
            !---------------------------------------
            if (LiqDirection<0.D0) then
               ZInterfLiqDens(I,J,K)=LiquidDens(I,J,K+1)
               ZInterfLiqSali(I,J,K)=LiquidSali(I,J,K+1)
               ZInterfLiqEnth(I,J,K)=LiquidEnth(I,J,K+1)
               ZInterfLiqVisc(I,J,K)=LiquidVisc(I,J,K+1)
               ZInterfLiqCp(I,J,K)=LiquidEnth(I,J,K+1)/Temp(I,J,K+1)
               ZInterfLiqVolSats(I,J,K)=LiqVolSats(I,J,K+1)
            elseif (LiqDirection>=0.D0) then
               ZInterfLiqDens(I,J,K)=LiquidDens(I,J,K)
               ZInterfLiqSali(I,J,K)=LiquidSali(I,J,K)
               ZInterfLiqEnth(I,J,K)=LiquidEnth(I,J,K)
               ZInterfLiqVisc(I,J,K)=LiquidVisc(I,J,K)
               ZInterfLiqCp(I,J,K)=LiquidEnth(I,J,K)/Temp(I,J,K)
               ZInterfLiqVolSats(I,J,K)=LiqVolSats(I,J,K)
            end if
            !---------------------------------------
            if (VapDirection<=0.D0) then
               ZInterfVapDens(I,J,K)=VaporDens(I,J,K+1)
               ZInterfVapSali(I,J,K)=VaporSali(I,J,K+1)
               ZInterfVapEnth(I,J,K)=VaporEnth(I,J,K+1)
               ZInterfVapVisc(I,J,K)=VaporVisc(I,J,K+1)
               ZInterfVapCp(I,J,K)=VaporEnth(I,J,K+1)/Temp(I,J,K+1)
               ZInterfVapVolSats(I,J,K)=VapVolSats(I,J,K+1)
            elseif (VapDirection>0.D0) then
               ZInterfVapDens(I,J,K)=VaporDens(I,J,K)
               ZInterfVapSali(I,J,K)=VaporSali(I,J,K)
               ZInterfVapEnth(I,J,K)=VaporEnth(I,J,K)
               ZInterfVapVisc(I,J,K)=VaporVisc(I,J,K)
               ZInterfVapCp(I,J,K)=VaporEnth(I,J,K)/Temp(I,J,K)
               ZInterfVapVolSats(I,J,K)=VapVolSats(I,J,K)
            end if
            !---------------------------------------
            ! these terms are needed for the diffusion part
            ! of the salt balance equation
            !---------------------------------------
            SaltLiqDiff = SaltChemDiff
            SaltVapDiff = 1.D4*SaltChemDiff
            ZInterfLiqDiff(I,J,K) = SaltLiqDiff*ZInterfPoros(I,J,K)* &
               ZInterfLiqVolSats(I,J,K)
            ZInterfVapDiff(I,J,K) = SaltVapDiff*ZInterfPoros(I,J,K)* &
               ZInterfVapVolSats(I,J,K)
            !---------------------------------------
         end do
      end do
   end do
end if
call StressInterface
end
