subroutine CornerDisp
! Purpose: Calculate diplacements at the corners of every control
!          volume via linear interpolation

use CommonVars, only: Prec,TWDispX,TEDispX,BWDispX,BEDispX,TWDispZ, &
   TEDispZ,BWDispZ,BEDispZ,NumNodesX,NumNodesY,NumNodesZ,DispX,     &
   DispZ,XNodeWidths,ZNodeWidths
implicit none
real(selected_real_kind(Prec)) :: CoordMat(4,3),x1,y1,x2,y2,x3,y3,  &
   x4,y4,phi1,phi2,phi3,phi4,x,y,phi
integer :: I,J,K

! Nomenclature: we set up a local coordinate system x,y with origin
! at the corner, then the nearest control volume nodes are labeled
! 1, 2, 3, and 4 as shown. x increases rightward and y increases
! upward.

! 2---------3 
! |         |    
! |  (0,0)  |   
! |         |  
! 1---------4   

x = 0.D0
y = 0.D0
do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ
         !-----------------------------------------------------------
         !                     top west corner
         !-----------------------------------------------------------
         if ((I==1).and.(K==1)) then
            ! There are no nodes 1 (W), 2 (WT), and 3 (T); set the 
            ! corner value to that at node 4 (P).
            TWDispX(I,J,K) = DispX(I,J,K)
            TWDispZ(I,J,K) = DispZ(I,J,K)             
         elseif ((I==1).and.(K>1)) then
            ! There are no nodes 1 (W) and 2 (WT); set the corner
            ! value to the average of nodes 3 (T) and 4 (P).
            x1 = -XNodeWidths(I)/2.D0
            y1 = -ZNodeWidths(K)/2.D0
            x2 = -XNodeWidths(I)/2.D0
            y2 =  ZNodeWidths(K-1)/2.D0
            x3 =  XNodeWidths(I)/2.D0
            y3 =  ZNodeWidths(K-1)/2.D0
            x4 =  XNodeWidths(I)/2.D0
            y4 = -ZNodeWidths(K)/2.D0
 
            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I,J,K)
            phi2 = DispX(I,J,K-1)
            phi3 = DispX(I,J,K-1)
            phi4 = DispX(I,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TWDispX(I,J,K))

            phi1 = DispZ(I,J,K)
            phi2 = DispZ(I,J,K-1)
            phi3 = DispZ(I,J,K-1)
            phi4 = DispZ(I,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TWDispZ(I,J,K))
         elseif ((I>1).and.(K==1)) then
            ! There are no nodes 2 (WT) and 3 (T); set the corner
            ! value to the average of nodes 1 (W) and 4 (P).
            x1 = -XNodeWidths(I-1)/2.D0
            y1 = -ZNodeWidths(K)/2.D0
            x2 = -XNodeWidths(I-1)/2.D0
            y2 =  ZNodeWidths(K)/2.D0
            x3 =  XNodeWidths(I)/2.D0
            y3 =  ZNodeWidths(K)/2.D0
            x4 =  XNodeWidths(I)/2.D0
            y4 = -ZNodeWidths(K)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I-1,J,K)
            phi2 = DispX(I-1,J,K)
            phi3 = DispX(I,J,K)
            phi4 = DispX(I,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TWDispX(I,J,K))

            phi1 = DispZ(I-1,J,K)
            phi2 = DispZ(I-1,J,K)
            phi3 = DispZ(I,J,K)
            phi4 = DispZ(I,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TWDispZ(I,J,K))
         else
            ! All surrounding nodes exist; use all four.
            x1 = -XNodeWidths(I-1)/2.D0
            y1 = -ZNodeWidths(K)/2.D0
            x2 = -XNodeWidths(I-1)/2.D0
            y2 =  ZNodeWidths(K-1)/2.D0
            x3 =  XNodeWidths(I)/2.D0
            y3 =  ZNodeWidths(K-1)/2.D0
            x4 =  XNodeWidths(I)/2.D0
            y4 = -ZNodeWidths(K)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I-1,J,K)
            phi2 = DispX(I-1,J,K-1)
            phi3 = DispX(I,J,K-1)
            phi4 = DispX(I,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TWDispX(I,J,K))

            phi1 = DispZ(I-1,J,K)
            phi2 = DispZ(I-1,J,K-1)
            phi3 = DispZ(I,J,K-1)
            phi4 = DispZ(I,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TWDispZ(I,J,K))
         end if
         !-----------------------------------------------------------
         !                     top east corner
         !-----------------------------------------------------------
         if ((I==NumNodesX).and.(K==1)) then
            ! There are no nodes 2 (T), 3 (TE), and 4 (E); set the
            ! corner value to that at node 1 (P).
            TEDispX(I,J,K) = DispX(I,J,K)
            TEDispZ(I,J,K) = DispZ(I,J,K)
         elseif ((I==NumNodesX).and.(K>1)) then
            ! There are no nodes 3 (TE) and 4 (E); set the corner
            ! to the average of nodes 1 (P) and 2 (T).
            x1 = -XNodeWidths(I)/2.D0
            y1 = -ZNodeWidths(K)/2.D0
            x2 = -XNodeWidths(I)/2.D0
            y2 =  ZNodeWidths(K-1)/2.D0
            x3 =  XNodeWidths(I)/2.D0
            y3 =  ZNodeWidths(K-1)/2.D0
            x4 =  XNodeWidths(I)/2.D0
            y4 = -ZNodeWidths(K)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I,J,K)
            phi2 = DispX(I,J,K-1)
            phi3 = DispX(I,J,K-1)
            phi4 = DispX(I,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TEDispX(I,J,K))

            phi1 = DispZ(I,J,K)
            phi2 = DispZ(I,J,K-1)
            phi3 = DispZ(I,J,K-1)
            phi4 = DispZ(I,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TEDispZ(I,J,K))
         elseif ((I<NumNodesX).and.(K==1)) then
            ! There are no nodes 2 (T) and 3 (TE); set the corner
            ! to the average of nodes 1 (P) and 4 (E).
            x1 = -XNodeWidths(I)/2.D0
            y1 = -ZNodeWidths(K)/2.D0
            x2 = -XNodeWidths(I)/2.D0
            y2 =  ZNodeWidths(K)/2.D0
            x3 =  XNodeWidths(I+1)/2.D0
            y3 =  ZNodeWidths(K)/2.D0
            x4 =  XNodeWidths(I+1)/2.D0
            y4 = -ZNodeWidths(K)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I,J,K)
            phi2 = DispX(I,J,K)
            phi3 = DispX(I+1,J,K)
            phi4 = DispX(I+1,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TEDispX(I,J,K))

            phi1 = DispZ(I,J,K)
            phi2 = DispZ(I,J,K)
            phi3 = DispZ(I+1,J,K)
            phi4 = DispZ(I+1,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TEDispZ(I,J,K))
         else
            ! All surrounding nodes exist; use all four.
            x1 = -XNodeWidths(I)/2.D0
            y1 = -ZNodeWidths(K)/2.D0
            x2 = -XNodeWidths(I)/2.D0
            y2 =  ZNodeWidths(K-1)/2.D0
            x3 =  XNodeWidths(I+1)/2.D0
            y3 =  ZNodeWidths(K-1)/2.D0
            x4 =  XNodeWidths(I+1)/2.D0
            y4 = -ZNodeWidths(K)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I,J,K)
            phi2 = DispX(I,J,K-1)
            phi3 = DispX(I+1,J,K-1)
            phi4 = DispX(I+1,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TEDispX(I,J,K))

            phi1 = DispZ(I,J,K)
            phi2 = DispZ(I,J,K-1)
            phi3 = DispZ(I+1,J,K-1)
            phi4 = DispZ(I+1,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,TEDispZ(I,J,K))
         end if
         !-----------------------------------------------------------
         !                     bottom west corner
         !-----------------------------------------------------------
         if ((I==1).and.(K==NumNodesZ)) then
            ! There are no nodes 1 (WB), 2 (W), and 4 (B); set the
            ! corner value to that at node 3 (P).
            BWDispX(I,J,K) = DispX(I,J,K)
            BWDispZ(I,J,K) = DispZ(I,J,K)
         elseif ((I==1).and.(K<NumNodesZ)) then
            ! There are no nodes 1 (WB) and 2 (W); set the corner
            ! to the average of nodes 3 (P) and 4 (B).
            x1 = -XNodeWidths(I)/2.D0
            y1 = -ZNodeWidths(K+1)/2.D0
            x2 = -XNodeWidths(I)/2.D0
            y2 =  ZNodeWidths(K)/2.D0
            x3 =  XNodeWidths(I)/2.D0
            y3 =  ZNodeWidths(K)/2.D0
            x4 =  XNodeWidths(I)/2.D0
            y4 = -ZNodeWidths(K+1)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I,J,K+1)
            phi2 = DispX(I,J,K)
            phi3 = DispX(I,J,K)
            phi4 = DispX(I,J,K+1)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BWDispX(I,J,K))

            phi1 = DispZ(I,J,K+1)
            phi2 = DispZ(I,J,K)
            phi3 = DispZ(I,J,K)
            phi4 = DispZ(I,J,K+1)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BWDispZ(I,J,K))
         elseif ((I>1).and.(K==NumNodesZ)) then
            ! There are no nodes 1 (WB) and 4 (B); set the corner
            ! to the average of nodes 2 (W) and 3 (P).
            x1 = -XNodeWidths(I-1)/2.D0
            y1 = -ZNodeWidths(K)/2.D0
            x2 = -XNodeWidths(I-1)/2.D0
            y2 =  ZNodeWidths(K)/2.D0
            x3 =  XNodeWidths(I)/2.D0
            y3 =  ZNodeWidths(K)/2.D0
            x4 =  XNodeWidths(I)/2.D0
            y4 = -ZNodeWidths(K)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I-1,J,K)
            phi2 = DispX(I-1,J,K)
            phi3 = DispX(I,J,K)
            phi4 = DispX(I,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BWDispX(I,J,K))

            phi1 = DispZ(I-1,J,K)
            phi2 = DispZ(I-1,J,K)
            phi3 = DispZ(I,J,K)
            phi4 = DispZ(I,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BWDispZ(I,J,K))
         else
            ! All surrounding nodes exist; use all four.
            x1 = -XNodeWidths(I-1)/2.D0
            y1 = -ZNodeWidths(K+1)/2.D0
            x2 = -XNodeWidths(I-1)/2.D0
            y2 =  ZNodeWidths(K)/2.D0
            x3 =  XNodeWidths(I)/2.D0
            y3 =  ZNodeWidths(K)/2.D0
            x4 =  XNodeWidths(I)/2.D0
            y4 = -ZNodeWidths(K+1)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I-1,J,K+1)
            phi2 = DispX(I-1,J,K)
            phi3 = DispX(I,J,K)
            phi4 = DispX(I,J,K+1)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BWDispX(I,J,K))

            phi1 = DispZ(I-1,J,K+1)
            phi2 = DispZ(I-1,J,K)
            phi3 = DispZ(I,J,K)
            phi4 = DispZ(I,J,K+1)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BWDispZ(I,J,K))
         end if
         !-----------------------------------------------------------
         !                     bottom east corner
         !-----------------------------------------------------------
         if ((I==NumNodesX).and.(K==NumNodesZ)) then
            ! There are no nodes 1 (B), 3 (E), and 4 (BE); set the
            ! corner value to that at node 2 (P).
            BEDispX(I,J,K) = DispX(I,J,K)
            BEDispZ(I,J,K) = DispZ(I,J,K)
         elseif ((I==NumNodesX).and.(K<NumNodesZ)) then
            ! There are no nodes 3 (E) and 4 (BE); set the corner
            ! to the average of nodes 1 (B) and 2 (P).
            x1 = -XNodeWidths(I)/2.D0
            y1 = -ZNodeWidths(K+1)/2.D0
            x2 = -XNodeWidths(I)/2.D0
            y2 =  ZNodeWidths(K)/2.D0
            x3 =  XNodeWidths(I)/2.D0
            y3 =  ZNodeWidths(K)/2.D0
            x4 =  XNodeWidths(I)/2.D0
            y4 = -ZNodeWidths(K+1)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I,J,K+1)
            phi2 = DispX(I,J,K)
            phi3 = DispX(I,J,K)
            phi4 = DispX(I,J,K+1)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BEDispX(I,J,K))

            phi1 = DispZ(I,J,K+1)
            phi2 = DispZ(I,J,K)
            phi3 = DispZ(I,J,K)
            phi4 = DispZ(I,J,K+1)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BEDispZ(I,J,K))
         elseif ((I<NumNodesX).and.(K==NumNodesZ)) then
            ! There are no nodes 1 (B) and 4 (BE); set the corner
            ! to the average of nodes 2 (P) and 3 (E).
            x1 = -XNodeWidths(I)/2.D0
            y1 = -ZNodeWidths(K)/2.D0
            x2 = -XNodeWidths(I)/2.D0
            y2 =  ZNodeWidths(K)/2.D0
            x3 =  XNodeWidths(I+1)/2.D0
            y3 =  ZNodeWidths(K)/2.D0
            x4 =  XNodeWidths(I+1)/2.D0
            y4 = -ZNodeWidths(K)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I,J,K)
            phi2 = DispX(I,J,K)
            phi3 = DispX(I+1,J,K)
            phi4 = DispX(I+1,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BEDispX(I,J,K))

            phi1 = DispZ(I,J,K)
            phi2 = DispZ(I,J,K)
            phi3 = DispZ(I+1,J,K)
            phi4 = DispZ(I+1,J,K)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BEDispZ(I,J,K))
         else
            ! All surrounding nodes exist; use all four.
            x1 = -XNodeWidths(I)/2.D0
            y1 = -ZNodeWidths(K+1)/2.D0
            x2 = -XNodeWidths(I)/2.D0
            y2 =  ZNodeWidths(K)/2.D0
            x3 =  XNodeWidths(I+1)/2.D0
            y3 =  ZNodeWidths(K)/2.D0
            x4 =  XNodeWidths(I+1)/2.D0
            y4 = -ZNodeWidths(K+1)/2.D0

            CoordMat(1,1) = x1
            CoordMat(1,2) = y1

            CoordMat(2,1) = x2
            CoordMat(2,2) = y2

            CoordMat(3,1) = x3
            CoordMat(3,2) = y3

            CoordMat(4,1) = x4
            CoordMat(4,2) = y4

            phi1 = DispX(I,J,K+1)
            phi2 = DispX(I,J,K)
            phi3 = DispX(I+1,J,K)
            phi4 = DispX(I+1,J,K+1)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BEDispX(I,J,K))

            phi1 = DispZ(I,J,K+1)
            phi2 = DispZ(I,J,K)
            phi3 = DispZ(I+1,J,K)
            phi4 = DispZ(I+1,J,K+1)

            CoordMat(1,3) = phi1
            CoordMat(2,3) = phi2
            CoordMat(3,3) = phi3
            CoordMat(4,3) = phi4
            call PlaneFit(CoordMat,x,y,BEDispZ(I,J,K))
         end if 
      end do
   end do
end do
end
