function DispXResid(NodeNumber)  result(Answer)
! purpose: calculate the local force balance residual in the
!          variable DispX
! input: NodeNumber = node at which to calculate the residual

use CommonVars, only: Prec,GridIndMat,XNodeWidths,ZNodeWidths,       &
   NumNodesX,NumNodesZ,StressGammaX,PoissonRatioX,DispX,BEDispZ,     &
   TEDispZ,BWDispZ,TWDispZ,StressNodeTypes,StressBoundVals,          &
   XNodeWidths,ZNodeWidths,StressBetaZ
implicit none
real(selected_real_kind(Prec)) :: Answer,NodeDist,Term1,Term2,Term3, &
   Term4,Term5,Term6,Term7,Term8
integer :: NodeNumber,I,J,K

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

if (StressNodeTypes(I,J,K)=='I') then
   NodeDist = XNodeWidths(I)/2.D0 + XNodeWidths(I+1)/2.D0
   Term1 = StressGammaX(I,J,K)*(1.D0-PoissonRatioX(I,J,K))* &
      (DispX(I+1,J,K)-DispX(I,J,K))/(XNodeWidths(I)*NodeDist)

   Term2 = StressGammaX(I,J,K)*PoissonRatioX(I,J,K)* &
      (BEDispZ(I,J,K)-TEDispZ(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

   NodeDist = XNodeWidths(I-1)/2.D0 + XNodeWidths(I)/2.D0
   Term3 = -StressGammaX(I-1,J,K)*(1.D0-PoissonRatioX(I-1,J,K))* &
      (DispX(I,J,K)-DispX(I-1,J,K))/(XNodeWidths(I)*NodeDist)

   Term4 = -StressGammaX(I-1,J,K)*PoissonRatioX(I-1,J,K)* &
      (BWDispZ(I,J,K)-TWDispZ(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

   NodeDist = ZNodeWidths(K)/2.D0 + ZNodeWidths(K+1)/2.D0
   Term5 = (StressBetaZ(I,J,K)/2.D0)*(DispX(I,J,K+1)-DispX(I,J,K))/ &
      (ZNodeWidths(K)*NodeDist)

   Term6 = (StressBetaZ(I,J,K)/2.D0)*(BEDispZ(I,J,K)-BWDispZ(I,J,K))/ &
      (XNodeWidths(I)*ZNodeWidths(K))

   NodeDist = ZNodeWidths(K-1)/2.D0 + ZNodeWidths(K)/2.D0
   Term7 = -(StressBetaZ(I,J,K-1)/2.D0)*(DispX(I,J,K)-DispX(I,J,K-1))/ &
      (ZNodeWidths(K)*NodeDist)

   Term8 = -(StressBetaZ(I,J,K-1)/2.D0)*(TEDispZ(I,J,K)-TWDispZ(I,J,K))/ &
      XNodeWidths(I)*ZNodeWidths(K)

   Answer = Term1 + Term2 + Term3 + Term4 + Term5 + Term6 + Term7 + &
            Term7 + Term8
   !write(*,*) 'DispX resid: '
   !write(*,*) Term1,Term2,Term3,Term4,Term5,Term6,Term7,Term8
   !write(*,*) '--------'
elseif (StressNodeTypes(I,J,K)=='R') then
   Answer = 0.D0
elseif (StressNodeTypes(I,J,K)=='S') then
   Answer = 0.D0
elseif (StressNodeTypes(I,J,K)=='P') then
   if (I==1) then
      ! West boundary
      NodeDist = XNodeWidths(I)/2.D0 + XNodeWidths(I+1)/2.D0
      Term1 = StressGammaX(I,J,K)*(1.D0-PoissonRatioX(I,J,K))* &
         (DispX(I+1,J,K)-DispX(I,J,K))/(XNodeWidths(I)*NodeDist)

      Term2 = StressGammaX(I,J,K)*PoissonRatioX(I,J,K)* &
         (BEDispZ(I,J,K)-TEDispZ(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

      Term3 = 0.D0

      Term4 = 0.D0

      if (K==NumNodesZ) then
         Term5 = 0.D0

         Term6 = 0.D0
      else
         NodeDist = ZNodeWidths(K)/2.D0 + ZNodeWidths(K+1)/2.D0
         Term5 = (StressBetaZ(I,J,K)/2.D0)*(DispX(I,J,K+1)-DispX(I,J,K))/ &
            (ZNodeWidths(K)*NodeDist)

         Term6 = (StressBetaZ(I,J,K)/2.D0)*(BEDispZ(I,J,K)-BWDispZ(I,J,K))/ &
            (XNodeWidths(I)*ZNodeWidths(K))
      end if

      if (K==1) then
         Term7 = 0.D0

         Term8 = 0.D0
      else
         NodeDist = ZNodeWidths(K-1)/2.D0 + ZNodeWidths(K)/2.D0
         Term7 = -(StressBetaZ(I,J,K-1)/2.D0)*(DispX(I,J,K)-DispX(I,J,K-1))/ &
            (ZNodeWidths(K)*NodeDist)

         Term8 = -(StressBetaZ(I,J,K-1)/2.D0)*(TEDispZ(I,J,K)-TWDispZ(I,J,K))/ &
            XNodeWidths(I)*ZNodeWidths(K)
      end if

      Answer = Term1 + Term2 + Term3 + Term4 + Term5 + Term6 + Term7 + &
               Term7 + Term8
   elseif (I==NumNodesX) then
      ! East boundary
      Term1 = 0.D0

      Term2 = 0.D0

      NodeDist = XNodeWidths(I-1)/2.D0 + XNodeWidths(I)/2.D0
      Term3 = -StressGammaX(I-1,J,K)*(1.D0-PoissonRatioX(I-1,J,K))* &
         (DispX(I,J,K)-DispX(I-1,J,K))/(XNodeWidths(I)*NodeDist)

      Term4 = -StressGammaX(I-1,J,K)*PoissonRatioX(I-1,J,K)* &
         (BWDispZ(I,J,K)-TWDispZ(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

      if (K==NumNodesZ) then
         Term5 = 0.D0
   
         Term6 = 0.D0
      else
         NodeDist = ZNodeWidths(K)/2.D0 + ZNodeWidths(K+1)/2.D0
         Term5 = (StressBetaZ(I,J,K)/2.D0)*(DispX(I,J,K+1)-DispX(I,J,K))/ &
            (ZNodeWidths(K)*NodeDist)

         Term6 = (StressBetaZ(I,J,K)/2.D0)*(BEDispZ(I,J,K)-BWDispZ(I,J,K))/ &
            (XNodeWidths(I)*ZNodeWidths(K))
      end if

      if (K==1) then
         Term7 = 0.D0

         Term8 = 0.D0
      else
         NodeDist = ZNodeWidths(K-1)/2.D0 + ZNodeWidths(K)/2.D0
         Term7 = -(StressBetaZ(I,J,K-1)/2.D0)*(DispX(I,J,K)-DispX(I,J,K-1))/ &
            (ZNodeWidths(K)*NodeDist)

         Term8 = -(StressBetaZ(I,J,K-1)/2.D0)*(TEDispZ(I,J,K)-TWDispZ(I,J,K))/ &
            XNodeWidths(I)*ZNodeWidths(K)
      end if

      Answer = Term1 + Term2 + Term3 + Term4 + Term5 + Term6 + Term7 + &
               Term7 + Term8
   elseif (K==1) then
      ! Top boundary
      if (I==NumNodesX) then
         Term1 = 0.D0

         Term2 = 0.D0
      else
         NodeDist = XNodeWidths(I)/2.D0 + XNodeWidths(I+1)/2.D0
         Term1 = StressGammaX(I,J,K)*(1.D0-PoissonRatioX(I,J,K))* &
            (DispX(I+1,J,K)-DispX(I,J,K))/(XNodeWidths(I)*NodeDist)

         Term2 = StressGammaX(I,J,K)*PoissonRatioX(I,J,K)* &
            (BEDispZ(I,J,K)-TEDispZ(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))
      end if

      if (I==1) then
         Term3 = 0.D0

         Term4 = 0.D0
      else
         NodeDist = XNodeWidths(I-1)/2.D0 + XNodeWidths(I)/2.D0
         Term3 = -StressGammaX(I-1,J,K)*(1.D0-PoissonRatioX(I-1,J,K))* &
            (DispX(I,J,K)-DispX(I-1,J,K))/(XNodeWidths(I)*NodeDist)

         Term4 = -StressGammaX(I-1,J,K)*PoissonRatioX(I-1,J,K)* &
            (BWDispZ(I,J,K)-TWDispZ(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))
      end if

      NodeDist = ZNodeWidths(K)/2.D0 + ZNodeWidths(K+1)/2.D0
      Term5 = (StressBetaZ(I,J,K)/2.D0)*(DispX(I,J,K+1)-DispX(I,J,K))/ &
         (ZNodeWidths(K)*NodeDist)

      Term6 = (StressBetaZ(I,J,K)/2.D0)*(BEDispZ(I,J,K)-BWDispZ(I,J,K))/ &
         (XNodeWidths(I)*ZNodeWidths(K))
   
      Term7 = 0.D0

      Term8 = 0.D0

      Answer = Term1 + Term2 + Term3 + Term4 + Term5 + Term6 + Term7 + &
               Term7 + Term8
   elseif (K==NumNodesZ) then
      ! Bottom boundary
      if (I==NumNodesX) then
         Term1 = 0.D0

         Term2 = 0.D0
      else
         NodeDist = XNodeWidths(I)/2.D0 + XNodeWidths(I+1)/2.D0
         Term1 = StressGammaX(I,J,K)*(1.D0-PoissonRatioX(I,J,K))* &
            (DispX(I+1,J,K)-DispX(I,J,K))/(XNodeWidths(I)*NodeDist)

         Term2 = StressGammaX(I,J,K)*PoissonRatioX(I,J,K)* &
            (BEDispZ(I,J,K)-TEDispZ(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))
      end if

      if (I==1) then
         Term3 = 0.D0

         Term4 = 0.D0
      else
         NodeDist = XNodeWidths(I-1)/2.D0 + XNodeWidths(I)/2.D0
         Term3 = -StressGammaX(I-1,J,K)*(1.D0-PoissonRatioX(I-1,J,K))* &
            (DispX(I,J,K)-DispX(I-1,J,K))/(XNodeWidths(I)*NodeDist)

         Term4 = -StressGammaX(I-1,J,K)*PoissonRatioX(I-1,J,K)* &
            (BWDispZ(I,J,K)-TWDispZ(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))
      end if

      Term5 = 0.D0

      Term6 = 0.D0

      NodeDist = ZNodeWidths(K-1)/2.D0 + ZNodeWidths(K)/2.D0
      Term7 = -(StressBetaZ(I,J,K-1)/2.D0)*(DispX(I,J,K)-DispX(I,J,K-1))/ &
         (ZNodeWidths(K)*NodeDist)

      Term8 = -(StressBetaZ(I,J,K-1)/2.D0)*(TEDispZ(I,J,K)-TWDispZ(I,J,K))/ &
         XNodeWidths(I)*ZNodeWidths(K)
 
      Answer = Term1 + Term2 + Term3 + Term4 + Term5 + Term6 + Term7 + &
               Term7 + Term8
   end if
end if
end
