function DispXResidDeriv(Node1,Node2,Primary) result(Answer)
! purpose: calculate the derivative of the force balance residual
!          in terms of the x displacement at node Node1 with respect 
!          to an increase in the x dispacement or z displacement at
!          node Node2
! input: Node1 = node at which residual is calculated
!        Node2 = node at which a variable is being increased
!        Primary = 'X','Z', which displacement variable to vary

use CommonVars, only: Prec,GridIndMat,NumNodesX,NumNodesZ, &
   XNodeWidths,ZNodeWidths,YoungModulusX,PoissonRatioX,    &
   YoungModulusZ,PoissonRatioZ,StressGammaX,StressGammaZ,  &
   StressBetaX,StressBetaZ
implicit none
real(selected_real_kind(Prec)) :: Answer,NodeDist,Term1,Term2, &
   Term3,Term4
character :: Primary,Direction,RelativeDirection
integer Node1,Node2,I,J,K

I = GridIndMat(Node1,1)
J = GridIndMat(Node1,2)
K = GridIndMat(Node1,3)

if (Node1==Node2) then
   if (Primary=='X') then
      if (I<NumNodesX) then
         NodeDist = XNodeWidths(I)/2.D0+XNodeWidths(I+1)/2.D0
         Term1 = -StressGammaX(I,J,K)*(1.D0-PoissonRatioX(I,J,K))/ &
                 (XNodeWidths(I)*NodeDist)
      else
         Term1 = 0.D0
      end if

      if (I>1) then
         NodeDist = XNodeWidths(I-1)/2.D0+XNodeWidths(I)/2.D0
         Term2 = -StressGammaX(I-1,J,K)*(1.D0-PoissonRatioX(I-1,J,K))/ &
                 (XNodeWidths(I)*NodeDist)
      else
         Term2 = 0.D0
      end if

      if (K<NumNodesZ) then
         NodeDist = ZNodeWidths(K)/2.D0+ZNodeWidths(K+1)/2.D0
         Term3 = -(StressBetaZ(I,J,K)/2.D0)/(ZNodeWidths(K)*NodeDist)
      else
         Term3 = 0.D0
      end if

      if (K>1) then
         NodeDist = ZNodeWidths(K-1)/2.D0+ZNodeWidths(K)/2.D0
         Term4 = -(StressBetaZ(I,J,K-1)/2.D0)/(ZNodeWidths(K)*NodeDist)
      else
         Term4 = 0.D0
      end if
      Answer = Term1 + Term2 + Term3 + Term4
   elseif (Primary=='Z') then
      Answer = 0.D0
   end if
else
   Direction = RelativeDirection(Node1,Node2)
   if (Direction=='W') then
      if (Primary=='X') then
         NodeDist = XNodeWidths(I-1)/2.D0+XNodeWidths(I)/2.D0
         Answer = StressGammaX(I-1,J,K)*(1.D0-PoissonRatioX(I-1,J,K))/ &
                  (XNodeWidths(I)*NodeDist)
      elseif (Primary=='Z') then
         Answer = 0.D0
      end if
   elseif (Direction=='E') then
      if (Primary=='X') then
         NodeDist = XNodeWidths(I)/2.D0+XNodeWidths(I+1)/2.D0
         Answer = StressGammaX(I,J,K)*(1.D0-PoissonRatioX(I,J,K))/ &
                  (XNodeWidths(I)*NodeDist)
      elseif (Primary=='Z') then
         Answer = 0.D0
      end if
   elseif (Direction=='T') then
      if (Primary=='X') then
         NodeDist = ZNodeWidths(K-1)/2.D0+ZNodeWidths(K)/2.D0
         Answer = (StressBetaZ(I,J,K-1)/2.D0)/(ZNodeWidths(K)*NodeDist)
      elseif (Primary=='Z') then
         Answer = 0.D0
      end if
   elseif (Direction=='B') then
      if (Primary=='X') then
         NodeDist = ZNodeWidths(K)/2.D0+ZNodeWidths(K+1)/2.D0
         Answer = (StressBetaZ(I,J,K)/2.D0)/(ZNodeWidths(K)*NodeDist)
      elseif (Primary=='Z') then
         Answer = 0.D0
      end if
   end if
end if
!write(*,*) Answer,I,J,K,Node1,Node2,Primary
!stop
end
