function DispZResid(NodeNumber)  result(Answer)
! purpose: calculate the local force balance residual in the
!          variable DispZ
! input: NodeNumber = node at which to calculate the residual

use CommonVars, only: Prec,GridIndMat,XNodeWidths,ZNodeWidths,       &
   NumNodesX,NumNodesZ,StressGammaZ,PoissonRatioZ,DispZ,BEDispX,     &
   TEDispX,BWDispX,TWDispX,StressNodeTypes,StressBoundVals,          &
   StressBetaX,RockDens,Grav
implicit none
real(selected_real_kind(Prec)) :: Answer,NodeDist,Term1,Term2,Term3, &
   Term4,Term5,Term6,Term7,Term8,Term9
integer :: NodeNumber,I,J,K

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

if (StressNodeTypes(I,J,K)=='I') then
   Term1 = (StressBetaX(I,J,K)/2.D0)* &
      (BEDispX(I,J,K)-TEDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

   NodeDist = XNodeWidths(I)/2.D0 + XNodeWidths(I+1)/2.D0
   Term2 = (StressBetaX(I,J,K)/2.D0)* &
      (DispZ(I+1,J,K)-DispZ(I,J,K))/(XNodeWidths(I)*NodeDist)

   Term3 = -(StressBetaX(I-1,J,K)/2.D0)* &
      (BWDispX(I,J,K)-TWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

   NodeDist = XNodeWidths(I-1)/2.D0 + XNodeWidths(I)/2.D0
   Term4 = -(StressBetaX(I-1,J,K)/2.D0)* &
      (DispZ(I,J,K)-DispZ(I-1,J,K))/(XNodeWidths(I)*NodeDist)

   NodeDist = ZNodeWidths(K)/2.D0 + ZNodeWidths(K+1)/2.D0
   Term5 = StressGammaZ(I,J,K)*(1.D0-PoissonRatioZ(I,J,K))* &
      (DispZ(I,J,K+1)-DispZ(I,J,K))/(ZNodeWidths(K)*NodeDist)
   
   Term6 = StressGammaZ(I,J,K)*PoissonRatioZ(I,J,K)* &
      (BEDispX(I,J,K)-BWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

   NodeDist = ZNodeWidths(K-1)/2.D0 + ZNodeWidths(K)/2.D0
   Term7 = -StressGammaZ(I,J,K-1)*(1.D0-PoissonRatioZ(I,J,K-1))* &
      (DispZ(I,J,K)-DispZ(I,J,K-1))/(ZNodeWidths(K)*NodeDist)

   Term8 = -StressGammaZ(I,J,K-1)*PoissonRatioZ(I,J,K-1)* &
      (TEDispX(I,J,K)-TWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

   Term9 = RockDens*Grav

   Answer = Term1 + Term2 + Term3 + Term4 + Term5 + Term6 + Term7 + &
            Term8 + Term9
   !write(*,*) 'DispZ Resid: '
   !write(*,*) Term1,Term2,Term3,Term4,Term5,Term6,Term7,Term8
   !write(*,*) '--------'
elseif (StressNodeTypes(I,J,K)=='R') then
   Answer = 0.D0
elseif (StressNodeTypes(I,J,K)=='S') then
   Answer = 0.D0
elseif (StressNodeTypes(I,J,K)=='P') then
   if (I==1) then
      ! west boundary
      Term1 = (StressBetaX(I,J,K)/2.D0)* &
         (BEDispX(I,J,K)-TEDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

      NodeDist = XNodeWidths(I)/2.D0 + XNodeWidths(I+1)/2.D0
      Term2 = (StressBetaX(I,J,K)/2.D0)* &
         (DispZ(I+1,J,K)-DispZ(I,J,K))/(XNodeWidths(I)*NodeDist)

      Term3 = 0.D0

      Term4 = 0.D0

      if (K==NumNodesZ) then
         Term5 = 0.D0

         Term6 = 0.D0
      else
         NodeDist = ZNodeWidths(K)/2.D0 + ZNodeWidths(K+1)/2.D0
         Term5 = StressGammaZ(I,J,K)*(1.D0-PoissonRatioZ(I,J,K))* &
            (DispZ(I,J,K+1)-DispZ(I,J,K))/(ZNodeWidths(K)*NodeDist)

         Term6 = StressGammaZ(I,J,K)*PoissonRatioZ(I,J,K)* &
            (BEDispX(I,J,K)-BWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))
      end if

      if (K==1) then
         Term7 = 0.D0
 
         Term8 = 0.D0
      else
         NodeDist = ZNodeWidths(K-1)/2.D0 + ZNodeWidths(K)/2.D0
         Term7 = -StressGammaZ(I,J,K-1)*(1.D0-PoissonRatioZ(I,J,K-1))* &
            (DispZ(I,J,K)-DispZ(I,J,K-1))/(ZNodeWidths(K)*NodeDist)
  
         Term8 = -StressGammaZ(I,J,K-1)*PoissonRatioZ(I,J,K-1)* &
            (TEDispX(I,J,K)-TWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))
      end if
 
      Term9 = RockDens*Grav

      Answer = Term1 + Term2 + Term3 + Term4 + Term5 + Term6 + Term7 + &
               Term8 + Term9
   elseif (I==NumNodesX) then
      ! east boundary
      Term1 = 0.D0
 
      Term2 = 0.D0

      Term3 = -(StressBetaX(I-1,J,K)/2.D0)* &
         (BWDispX(I,J,K)-TWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

      NodeDist = XNodeWidths(I-1)/2.D0 + XNodeWidths(I)/2.D0
      Term4 = -(StressBetaX(I-1,J,K)/2.D0)* &
         (DispZ(I,J,K)-DispZ(I-1,J,K))/(XNodeWidths(I)*NodeDist)

      if (K==NumNodesZ) then
         Term5 = 0.D0

         Term6 = 0.D0
      else
         NodeDist = ZNodeWidths(K)/2.D0 + ZNodeWidths(K+1)/2.D0
         Term5 = StressGammaZ(I,J,K)*(1.D0-PoissonRatioZ(I,J,K))* &
            (DispZ(I,J,K+1)-DispZ(I,J,K))/(ZNodeWidths(K)*NodeDist)

         Term6 = StressGammaZ(I,J,K)*PoissonRatioZ(I,J,K)* &
            (BEDispX(I,J,K)-BWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))
      end if

      if (K==1) then
         Term7 = 0.D0

         Term8 = 0.D0
      else
         NodeDist = ZNodeWidths(K-1)/2.D0 + ZNodeWidths(K)/2.D0
         Term7 = -StressGammaZ(I,J,K-1)*(1.D0-PoissonRatioZ(I,J,K-1))* &
            (DispZ(I,J,K)-DispZ(I,J,K-1))/(ZNodeWidths(K)*NodeDist)

         Term8 = -StressGammaZ(I,J,K-1)*PoissonRatioZ(I,J,K-1)* &
            (TEDispX(I,J,K)-TWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))
      end if

      Term9 = RockDens*Grav

      Answer = Term1 + Term2 + Term3 + Term4 + Term5 + Term6 + Term7 + &
               Term8 + Term9
   elseif (K==1) then
      ! top boundary
      if (I==NumNodesX) then
         Term1 = 0.D0
      
         Term2 = 0.D0
      else
         Term1 = (StressBetaX(I,J,K)/2.D0)* &
            (BEDispX(I,J,K)-TEDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))
   
         NodeDist = XNodeWidths(I)/2.D0 + XNodeWidths(I+1)/2.D0
         Term2 = (StressBetaX(I,J,K)/2.D0)* &
            (DispZ(I+1,J,K)-DispZ(I,J,K))/(XNodeWidths(I)*NodeDist)
      end if

      if (I==1) then
         Term3 = 0.D0

         Term4 = 0.D0
      else
         Term3 = -(StressBetaX(I-1,J,K)/2.D0)* &
            (BWDispX(I,J,K)-TWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

         NodeDist = XNodeWidths(I-1)/2.D0 + XNodeWidths(I)/2.D0
         Term4 = -(StressBetaX(I-1,J,K)/2.D0)* &
            (DispZ(I,J,K)-DispZ(I-1,J,K))/(XNodeWidths(I)*NodeDist)
      end if

      NodeDist = ZNodeWidths(K)/2.D0 + ZNodeWidths(K+1)/2.D0
      Term5 = StressGammaZ(I,J,K)*(1.D0-PoissonRatioZ(I,J,K))* &
         (DispZ(I,J,K+1)-DispZ(I,J,K))/(ZNodeWidths(K)*NodeDist)

      Term6 = StressGammaZ(I,J,K)*PoissonRatioZ(I,J,K)* &
         (BEDispX(I,J,K)-BWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

      Term7 = 0.D0
 
      Term8 = 0.D0

      Term9 = RockDens*Grav
  
      Answer = Term1 + Term2 + Term3 + Term4 + Term5 + Term6 + Term7 + &
               Term8 + Term9
   elseif (K==NumNodesZ) then
      ! bottom boundary
      if (I==NumNodesX) then
         Term1 = 0.D0

         Term2 = 0.D0
      else
         Term1 = (StressBetaX(I,J,K)/2.D0)* &
            (BEDispX(I,J,K)-TEDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

         NodeDist = XNodeWidths(I)/2.D0 + XNodeWidths(I+1)/2.D0
         Term2 = (StressBetaX(I,J,K)/2.D0)* &
            (DispZ(I+1,J,K)-DispZ(I,J,K))/(XNodeWidths(I)*NodeDist)
      end if

      if (I==1) then
         Term3 = 0.D0
      
         Term4 = 0.D0
      else
         Term3 = -(StressBetaX(I-1,J,K)/2.D0)* &
            (BWDispX(I,J,K)-TWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

         NodeDist = XNodeWidths(I-1)/2.D0 + XNodeWidths(I)/2.D0
         Term4 = -(StressBetaX(I-1,J,K)/2.D0)* &
            (DispZ(I,J,K)-DispZ(I-1,J,K))/(XNodeWidths(I)*NodeDist)
      end if

      Term5 = 0.D0

      Term6 = 0.D0

      NodeDist = ZNodeWidths(K-1)/2.D0 + ZNodeWidths(K)/2.D0
      Term7 = -StressGammaZ(I,J,K-1)*(1.D0-PoissonRatioZ(I,J,K-1))* &
         (DispZ(I,J,K)-DispZ(I,J,K-1))/(ZNodeWidths(K)*NodeDist)

      Term8 = -StressGammaZ(I,J,K-1)*PoissonRatioZ(I,J,K-1)* &
         (TEDispX(I,J,K)-TWDispX(I,J,K))/(XNodeWidths(I)*ZNodeWidths(K))

      Term9 = RockDens*Grav

      Answer = Term1 + Term2 + Term3 + Term4 + Term5 + Term6 + Term7 + &
               Term8 + Term9
   end if
end if
end
