function EnergyAccum(NodeNumber,Tempr,Presr,Salin,Which,Linear) &
   result(Answer)
! purpose: calculate the term of which the time derivative is
!          taken in the energy balance equation (phi*rho*h + ...)
! input: NodeNumber = node at which to calculate this value
!        Tempr = temperature(C); Presr = pressure (Pa);
!        Salin = salinity(wt.fraction NaCl)
!        Which = R or F for Rock or Fluid
!        Linear = 'Y' if the linearized bulk enthalpy and bulk
!                  density are to be used; 'N' otherwise

use CommonVars, only: Prec,RockDens,CpRock,GridIndMat,LogFile
implicit none
real(selected_real_kind(Prec)) :: Tempr,Presr,Salin,Answer,CalcPoros, &
   CalcLinrDens,CalcLinrEnth,Term1,Term2,Term3,Poros,CalcBulkDens,    &
   CalcBulkEnth
character :: Linear,Which
integer :: NodeNumber,I,J,K

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

Poros = CalcPoros(NodeNumber,Tempr,Presr,Salin)
if (Linear=='Y') then
   Term1 = Poros*CalcLinrDens(I,J,K,Tempr,Presr)* &
                 CalcLinrEnth(I,J,K,Tempr,Presr)
elseif (Linear=='N') then
   Term1 = Poros*CalcBulkDens(Tempr,Presr,Salin)* &
                 CalcBulkEnth(Tempr,Presr,Salin)
else
   write(LogFile,*) 'Variable Linear not set - MassAccum.f90'
   write(LogFile,*) 'Terminating.'
   stop
end if
Term2 = -Poros*Presr
Term3 = (1.D0-Poros)*RockDens*CpRock*Tempr
if (Which=='B') then
   Answer = Term1+Term2+Term3
elseif (Which=='F') then
   Answer = Term1+Term2
elseif (Which=='R') then
   Answer = Term3
end if
end
