function EnergyIn(I,J,K,FluxType,Face) Result(Answer)
! inputs: three node indices, FluxType = 'A' for the advective flux, 
!         FluxType = 'D' for the diffusive flux, or FluxType = 'B'
!         for the sum of both; Face = W,E,T,B for the cell face at 
!         which the flux is desired. 
!         W = West, E = East, T = Top, B = Bottom
! output: energy flux (W/m^2) in the z-direction (downward)

use CommonVars, only: Prec,XNodeWidths,YNodeWidths,ZNodeWidths,      &
   Pres,Sali,RockDens,CpRock,Porosities,Delta_t,XInterfHeatFlux,     &
   ZInterfHeatFlux,XInterfPoros,ZInterfPoros,FluidThermCond,         &
   RockThermCond,TempNodeTypes,TempBoundVals,NumNodesX,NumNodesZ,    &
   Temp,TempGamma1Z,TempGamma2Z,ZInterfEffCond,TempGamma1X,          &
   XInterfEffCond
implicit none
real(selected_real_kind(Prec)) :: Answer,CrossArea,NodeDistance,     &
   KMedium,EnergyInWest,EnergyInEast,EnergyInTop,EnergyInBot,        &
   EnergyDiffWest,EnergyDiffEast,EnergyDiffTop,EnergyDiffBot,        &
   CurrentEnergyIn,EnergyErr,EnergyDiffTop2,EnergyDiffBot2
character :: Face,FluxType
integer :: I,J,K

if ((I<1).or.(I>NumNodesX).or.(K<1).or.(K>NumNodesZ)) then
   Answer = 0.D0
else
!----------
if (Face=='W') then
   CrossArea  = YNodeWidths(J)*ZNodeWidths(K)
   if (I > 1) then
      NodeDistance = (XNodeWidths(I)+XNodeWidths(I-1))/2.D0
      EnergyInWest = (-TempGamma1X(I-1,J,K)*Pres(I,J,K)/NodeDistance + &
                      TempGamma1X(I-1,J,K)*Pres(I-1,J,K)/NodeDistance)* &
                      CrossArea*Delta_t

      EnergyDiffWest = (XInterfEffCond(I-1,J,K)*Temp(I-1,J,K)/ &
                       NodeDistance - &
                       XInterfEffCond(I-1,J,K)*Temp(I,J,K)/ &
                       NodeDistance)*CrossArea*Delta_t
   else
      if ((TempNodeTypes(I,J,K)=='F').and.(NumNodesX>1)) then
         EnergyInWest = TempBoundVals(I,J,K)*CrossArea*Delta_t
         EnergyDiffWest = 0.D0
      else
         EnergyInWest = 0.D0
         EnergyDiffWest = 0.D0
      end if
   end if
   if (FluxType=='B') then
      Answer = EnergyInWest + EnergyDiffWest
   elseif (FluxType=='A') then
      Answer = EnergyInWest
   elseif (FluxType=='D') then
      Answer = EnergyDiffWest
   end if
elseif (Face=='E') then
   CrossArea  = YNodeWidths(J)*ZNodeWidths(K)
   if (I < NumNodesX) then
      NodeDistance = (XNodeWidths(I)+XNodeWidths(I+1))/2.D0
      EnergyInEast = (TempGamma1X(I,J,K)*Pres(I+1,J,K)/NodeDistance - &
                     TempGamma1X(I,J,K)*Pres(I,J,K)/NodeDistance)* &
                     CrossArea*Delta_t
      EnergyDiffEast = (XInterfEffCond(I,J,K)*Temp(I+1,J,K)/ &
                       NodeDistance - &
                       XInterfEffCond(I,J,K)*Temp(I,J,K)/ &
                       NodeDistance)*CrossArea*Delta_t
   else
      if ((TempNodeTypes(I,J,K)=='F').and.(NumNodesX>1)) then
         EnergyInEast = TempBoundVals(I,J,K)*CrossArea*Delta_t
         EnergyDiffEast = 0.D0
      else
         EnergyInEast = 0.D0
         EnergyDiffEast = 0.D0
      end if
   end if
   if (FluxType=='B') then
      Answer = EnergyInEast + EnergyDiffEast
   elseif (FluxType=='A') then
      Answer = EnergyInEast
   elseif (FluxType=='D') then
      Answer = EnergyDiffEast
   end if
elseif (Face=='T') then
   CrossArea  = YNodeWidths(J)*XNodeWidths(I)
   if (K > 1) then
      NodeDistance = (ZNodeWidths(K)+ZNodeWidths(K-1))/2.D0
      EnergyInTop = (-TempGamma1Z(I,J,K-1)*Pres(I,J,K)/NodeDistance + &
                      TempGamma1Z(I,J,K-1)*Pres(I,J,K-1)/NodeDistance + &
                      TempGamma2Z(I,J,K-1))*CrossArea*Delta_t
      EnergyDiffTop = (ZInterfEffCond(I,J,K-1)*Temp(I,J,K-1)/ &
                       NodeDistance - &
                       ZInterfEffCond(I,J,K-1)*Temp(I,J,K)/ &
                       NodeDistance)*CrossArea*Delta_t
   else
      if ((TempNodeTypes(I,J,K)=='F').and.(NumNodesZ>1)) then
         EnergyInTop = TempBoundVals(I,J,K)*CrossArea*Delta_t
         EnergyDiffTop = 0.D0
      else
         EnergyInTop = 0.D0
         EnergyDiffTop = 0.D0
      end if
   end if
   if (FluxType=='B') then
      Answer = EnergyInTop + EnergyDiffTop
   elseif (FluxType=='A') then
      Answer = EnergyInTop
   elseif (FluxType=='D') then
      Answer = EnergyDiffTop
   end if
elseif (Face=='B') then
   CrossArea  = YNodeWidths(J)*XNodeWidths(I)
   if (K < NumNodesZ) then
      NodeDistance = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
      EnergyInBot = (TempGamma1Z(I,J,K)*Pres(I,J,K+1)/NodeDistance - &
                     TempGamma1Z(I,J,K)*Pres(I,J,K)/NodeDistance - &
                     TempGamma2Z(I,J,K))*CrossArea*Delta_t
      EnergyDiffBot = (ZInterfEffCond(I,J,K)*Temp(I,J,K+1)/ &
                       NodeDistance - &
                       ZInterfEffCond(I,J,K)*Temp(I,J,K)/ &
                       NodeDistance)*CrossArea*Delta_t
   else
      if ((TempNodeTypes(I,J,K)=='F').and.(NumNodesZ>1)) then
         EnergyInBot = TempBoundVals(I,J,K)*CrossArea*Delta_t
         EnergyDiffBot = 0.D0
      else
         EnergyInBot = 0.D0
         EnergyDiffBot = 0.D0
      end if
   end if
   if (FluxType=='B') then
      Answer = EnergyInBot + EnergyDiffBot
   elseif (FluxType=='A') then
      Answer = EnergyInBot
   elseif (FluxType=='D') then
      Answer = EnergyDiffBot
   end if
end if
!----------
end if
end
