FUNCTION FViscosity(Tempr,Presr,Salin)
! Purpose: To calculate the the dynamic viscosity (Pa*s) of 
!          the fluid (liquid and vapor).
! Input: Temperature (C), pressure (Pa), salinity (wt.
!        fraction NaCl.
! Output: Fluid viscosity (Pa*s)

use CommonVars, only: Prec,LogFile
implicit none
real (selected_real_kind(Prec)) :: Tempr,Presr,Salin,             &
     PureWaterDens,PureWaterVisc,SVisc,T,FViscosity,A1,A2,        &
     PWViscosity,CalcBulkDens,PWSali,B1,B2,B3,e1,e2,EquivTempr
data A1/-35.9858/, A2/0.80017/, B1 /1.D-6/, B2 /-0.05239/,        &
     B3 /1.32936/, PWSali/0.D0/

e1 = A1*(Salin**A2)
e2 = 1.0-B1*(Tempr**B2)-B3*(Salin**A2)*(Tempr**B2)
EquivTempr = e1+e2*Tempr

PureWaterDens = CalcBulkDens(EquivTempr,Presr,Salin)
if (PureWaterDens==-1.D0) then
   write(LogFile,*) 'CalcBulkDens returned an error -FViscosity'
   write(LogFile,*) 'Halting'
   stop
end if

FViscosity = PWViscosity(EquivTempr,PureWaterDens)
end
