function InterfDens(NodeNumber,Face,Phase,Tempr,Presr,Salin) result(Answer)
! Purpose: calculate the liquid or vapor density on a control volume
!          interface, without upwinding (for the buoyancy terms)
! Input: NodeNumber, the number of the node whose interface value is
!        to be obtained; Face = 'W','E','T','B' indicating which face;
!        and Phase = 'L','V' indicating liquid or vapor density
!        Tempr,Presr,Salin are the temperature(C), pressure (Pa), and
!        salinity (wt.fraction NaCl) at node NodeNumber

use CommonVars, only: Prec,GridIndMat,LogFile,NumNodesX,NumNodesY, &
   NumNodesZ,XNodeWidths,ZNodeWidths,Temp,Pres,Sali
implicit none
real(selected_real_kind(Prec)) :: Answer,NodeDist,RelInterfPos, &
   ThermProp,Dens1,Dens2,Tempr,Presr,Salin
character :: Face,Phase,Prop
integer :: NodeNumber,I,J,K,NodeExist,ChkNeighborExist

NodeExist = ChkNeighborExist(NodeNumber,Face)

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

if (NodeExist==1) then
   Prop = 'D'
   if (Face=='W') then
      NodeDist = XNodeWidths(I)/2.D0 + &
                 XNodeWidths(I-1)/2.D0
      RelInterfPos = (XNodeWidths(I-1)/2.D0)/NodeDist
      Dens1 = ThermProp(Temp(I-1,J,K),Pres(I-1,J,K),Sali(I-1,J,K), &
                        Phase,Prop)
      Dens2 = ThermProp(Tempr,Presr,Salin,Phase,Prop)
      Answer = (1.D0-RelInterfPos)*Dens1 + RelInterfPos*Dens2
   elseif (Face=='E') then
      NodeDist = XNodeWidths(I)/2.D0 + &
                 XNodeWidths(I+1)/2.D0
      RelInterfPos = (XNodeWidths(I)/2.D0)/NodeDist
      Dens1 = ThermProp(Tempr,Presr,Salin,Phase,Prop)
      Dens2 = ThermProp(Temp(I+1,J,K),Pres(I+1,J,K),Sali(I+1,J,K), &
                        Phase,Prop)
      Answer = (1.D0-RelInterfPos)*Dens1 + RelInterfPos*Dens2
   elseif (Face=='T') then
      NodeDist = ZNodeWidths(K)/2.D0 + &
                 ZNodeWidths(K-1)/2.D0
      RelInterfPos = (ZNodeWidths(K-1)/2.D0)/NodeDist
      Dens1 = ThermProp(Temp(I,J,K-1),Pres(I,J,K-1),Sali(I,J,K-1), &
                        Phase,Prop)
      Dens2 = ThermProp(Tempr,Presr,Salin,Phase,Prop)
      Answer = (1.D0-RelInterfPos)*Dens1 + RelInterfPos*Dens2
   elseif (Face=='B') then
      NodeDist = ZNodeWidths(K)/2.D0 + &
                 ZNodeWidths(K+1)/2.D0
      RelInterfPos = (ZNodeWidths(K)/2.D0)/NodeDist
      Dens1 = ThermProp(Tempr,Presr,Salin,Phase,Prop)
      Dens2 = ThermProp(Temp(I,J,K+1),Pres(I,J,K+1),Sali(I,J,K+1), &
                        Phase,Prop)
      Answer = (1.D0-RelInterfPos)*Dens1 + RelInterfPos*Dens2
   end if
else
   write(LogFile,*) 'Specified face is out of bounds -InterfDens'
   write(LogFile,*) 'Node: ',I,J,K,'Face: ',Face
   stop
end if
end
