function InterfDens1(NodeNumber,Face,Phase) result(Answer)
! Purpose: calculate the liquid or vapor density on a control volume
!          interface, without upwinding (for the buoyancy terms)
! Input: NodeNumber, the number of the node whose interface value is
!        to be obtained; Face = 'W','E','T','B' indicating which face;
!        and Phase = 'L','V' indicating liquid or vapor density

use CommonVars, only: Prec,GridIndMat,LogFile,NumNodesX,NumNodesY, &
   NumNodesZ,XNodeWidths,ZNodeWidths
implicit none
real(selected_real_kind(Prec)) :: Answer,NodeDist,RelInterfPos
character :: Face,Phase
integer :: NodeNumber,I,J,K,ErrFlag

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

ErrFlag = 0
if ((Face=='W').and.(I==1)) then
   ErrFlag = 1
elseif ((Face=='E').and.(I==NumNodesX)) then
   ErrFlag = 1
elseif ((Face=='T').and.(K==1)) then
   ErrFlag = 1
elseif ((Face=='B').and.(K==NumNodesZ)) then
   ErrFlag = 1
end if

if (ErrFlag==0) then
   if (Face=='W') then
      NodeDist = XNodeWidths(I)/2.D0 + &
                 XNodeWidths(I-1)/2.D0
      RelInterfPos = (XNodeWidths(I-1)/2.D0)/NodeDist
      if (Phase=='L') then
      elseif (Phase=='V') then
      end if
   elseif (Face=='E') then
      NodeDist = XNodeWidths(I)/2.D0 + &
                 XNodeWidths(I+1)/2.D0
      RelInterfPos = (XNodeWidths(I)/2.D0)/NodeDist
      if (Phase=='L') then
      elseif (Phase=='V') then
      end if
   elseif (Face=='T') then
      NodeDist = ZNodeWidths(K)/2.D0 + &
                 ZNodeWidths(K-1)/2.D0
      RelInterfPos = (ZNodeWidths(K-1)/2.D0)/NodeDist
      if (Phase=='L') then
      elseif (Phase=='V') then
      end if
   elseif (Face=='B') then
      NodeDist = ZNodeWidths(K)/2.D0 + &
                 ZNodeWidths(K+1)/2.D0
      RelInterfPos = (ZNodeWidths(K)/2.D0)/NodeDist
      if (Phase=='L') then
      elseif (Phase=='V') then
      end if
   end if
else
   write(LogFile,*) 'Specified face is out of bounds -InterfDens1'
   write(LogFile,*) 'Node: ',I,J,K,'Face: ',Face
   stop
end if

Answer = 0.0
end
