function InterfHaliteSat(NodeNumber,Face,Tempr,Presr,Salin) result(Answer)
! purpose: to calculate the halite volume saturation on a control
!          volume interface. 
! input: number of the node on whose interface to calculate;
!        Face = 'W','E','T','B' for the West, East, Top, or Bottom
!                interface
!        Tempr,Presr,Salin are the temperature(C), pressure (Pa), and
!        salinity (wt.fraction NaCl) at node NodeNumber

use CommonVars, only: Prec,LogFile,GridIndMat,XNodeWidths,ZNodeWidths, &
   Temp,Pres,Sali
implicit none
real(selected_real_kind(Prec)) :: Answer,HalSat1,HalSat2,CalcHaliteSat, &
   Tempr,Presr,Salin,NodeDist,RelInterfPos
character :: Face
integer :: NodeNumber,ChkNeighborExist,NodeExist,I,J,K

NodeExist = ChkNeighborExist(NodeNumber,Face)

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

if (NodeExist==1) then
   if (Face=='W') then
      NodeDist = XNodeWidths(I)/2.D0 + &
                 XNodeWidths(I-1)/2.D0
      RelInterfPos = (XNodeWidths(I-1)/2.D0)/NodeDist
      HalSat1 = CalcHaliteSat(Temp(I-1,J,K),Pres(I-1,J,K),Sali(I-1,J,K))
      HalSat2 = CalcHaliteSat(Tempr,Presr,Salin)
      Answer = (1.D0-RelInterfPos)*HalSat1 + RelInterfPos*HalSat2
   elseif (Face=='E') then
      NodeDist = XNodeWidths(I)/2.D0 + &
                 XNodeWidths(I+1)/2.D0
      RelInterfPos = (XNodeWidths(I)/2.D0)/NodeDist
      HalSat1 = CalcHaliteSat(Tempr,Presr,Salin)
      HalSat2 = CalcHaliteSat(Temp(I+1,J,K),Pres(I+1,J,K),Sali(I+1,J,K))
      Answer = (1.D0-RelInterfPos)*HalSat1 + RelInterfPos*HalSat2
   elseif (Face=='T') then
      NodeDist = ZNodeWidths(K)/2.D0 + &
                 ZNodeWidths(K-1)/2.D0
      RelInterfPos = (ZNodeWidths(K-1)/2.D0)/NodeDist
      HalSat1 = CalcHaliteSat(Temp(I,J,K-1),Pres(I,J,K-1),Sali(I,J,K-1))
      HalSat2 = CalcHaliteSat(Tempr,Presr,Salin)
      Answer = (1.D0-RelInterfPos)*HalSat1 + RelInterfPos*HalSat2
   elseif (Face=='B') then
      NodeDist = ZNodeWidths(K)/2.D0 + &
                 ZNodeWidths(K+1)/2.D0
      RelInterfPos = (ZNodeWidths(K)/2.D0)/NodeDist
      HalSat1 = CalcHaliteSat(Tempr,Presr,Salin)
      HalSat2 = CalcHaliteSat(Temp(I,J,K+1),Pres(I,J,K+1),Sali(I,J,K+1))
      Answer = (1.D0-RelInterfPos)*HalSat1 + RelInterfPos*HalSat2
   end if
else
   write(LogFile,*) 'the requested node does not exist! -InterfHaliteSat'
   write(LogFile,*) 'Node: ',I,J,K,'Face: ',Face
   stop
end if
end
