function InterfHeatFlux2(NodeNumber,Face,Phase,Tempr,Presr,Salin) &
                        result(Answer)

use CommonVars, only: Prec,GridIndMat,LogFile,XNodeWidths,ZNodeWidths, &
   Pres,Temp,Sali,InterfPermsX,InterfPermsZ,Grav,RockThermCond,        &
   XInterfLiqDens,XInterfLiqEnth,XInterfLiqVisc,XInterfLiqDens0,     &
   XInterfVapDens,XInterfVapEnth,XInterfVapVisc,XInterfVapDens0,     &
   ZInterfLiqDens,ZInterfLiqEnth,ZInterfLiqVisc,ZInterfLiqDens0,     &
   ZInterfVapDens,ZInterfVapEnth,ZInterfVapVisc,ZInterfVapDens0,     &
   XInterfLiqVolSats,ZInterfLiqVolSats,XInterfVapVolSats,            &
   ZInterfVapVolSats

implicit none
real(selected_real_kind(Prec)) :: Tempr,Presr,Salin,Answer,PresGrad, &
   NodeDist,PreFactor,RelPerm,Dens,Visc,InterfThermProp,GravTerm,    &
   CentralDiffDens,InterfDens,Enth,TempGrad
integer :: NodeNumber,I,J,K,NodeExist,ChkNeighborExist
character :: Face,Phase

NodeExist = ChkNeighborExist(NodeNumber,Face)

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

if (NodeExist==1) then
   if (Face=='W') then
      NodeDist = XNodeWidths(I)/2.D0 + &
                 XNodeWidths(I-1)/2.D0
      PresGrad = (Presr-Pres(I-1,J,K))/NodeDist
      TempGrad = (Tempr-Temp(I-1,J,K))/NodeDist

      if (Phase=='L') then
         RelPerm = XInterfLiqVolSats(I-1,J,K)
         Dens    = XInterfLiqDens(I-1,J,K)
         Enth    = XInterfLiqEnth(I-1,J,K)
         Visc    = XInterfLiqVisc(I-1,J,K)
      elseif (Phase=='V') then
         RelPerm = XInterfVapVolSats(I-1,J,K)
         Dens    = XInterfVapDens(I-1,J,K)
         Enth    = XInterfVapEnth(I-1,J,K)
         Visc    = XInterfVapVisc(I-1,J,K)
      end if

      PreFactor = InterfPermsX(I-1,J,K)*RelPerm*Dens*Enth/Visc
      Answer = -PreFactor*PresGrad
   elseif (Face=='E') then
      NodeDist = XNodeWidths(I)/2.D0 + &
                 XNodeWidths(I+1)/2.D0
      PresGrad = (Pres(I+1,J,K)-Presr)/NodeDist
      TempGrad = (Temp(I+1,J,K)-Tempr)/NodeDist

      if (Phase=='L') then
         RelPerm = XInterfLiqVolSats(I,J,K)
         Dens    = XInterfLiqDens(I,J,K)
         Enth    = XInterfLiqEnth(I,J,K)
         Visc    = XInterfLiqVisc(I,J,K)
      elseif (Phase=='V') then
         RelPerm = XInterfVapVolSats(I,J,K)
         Dens    = XInterfVapDens(I,J,K)
         Enth    = XInterfVapEnth(I,J,K)
         Visc    = XInterfVapVisc(I,J,K)
      end if
      PreFactor = InterfPermsX(I,J,K)*RelPerm*Dens*Enth/Visc
      Answer = -PreFactor*PresGrad
   elseif (Face=='T') then
      NodeDist = ZNodeWidths(K)/2.D0 + &
                 ZNodeWidths(K-1)/2.D0
      PresGrad = (Presr-Pres(I,J,K-1))/NodeDist
      TempGrad = (Tempr-Temp(I,J,K-1))/NodeDist

      if (Phase=='L') then
         RelPerm = ZInterfLiqVolSats(I,J,K-1)
         Dens    = ZInterfLiqDens(I,J,K-1)
         Enth    = ZInterfLiqEnth(I,J,K-1)
         Visc    = ZInterfLiqVisc(I,J,K-1)
         CentralDiffDens = ZInterfLiqDens0(I,J,K-1)
      elseif (Phase=='V') then
         RelPerm = ZInterfVapVolSats(I,J,K-1)
         Dens    = ZInterfVapDens(I,J,K-1)
         Enth    = ZInterfVapEnth(I,J,K-1)
         Visc    = ZInterfVapVisc(I,J,K-1)
         CentralDiffDens = ZInterfVapDens0(I,J,K-1)
      end if

      PreFactor = InterfPermsZ(I,J,K-1)*RelPerm*Dens*Enth/Visc
      GravTerm  = Grav*CentralDiffDens
      Answer = -PreFactor*(PresGrad - GravTerm)
   elseif (Face=='B') then
      NodeDist = ZNodeWidths(K)/2.D0 + &
                 ZNodeWidths(K+1)/2.D0
      PresGrad = (Pres(I,J,K+1)-Presr)/NodeDist
      TempGrad = (Temp(I,J,K+1)-Tempr)/NodeDist

      if (Phase=='L') then
         RelPerm = ZInterfLiqVolSats(I,J,K)
         Dens    = ZInterfLiqDens(I,J,K)
         Enth    = ZInterfLiqEnth(I,J,K)
         Visc    = ZInterfLiqVisc(I,J,K)
         CentralDiffDens = ZInterfLiqDens0(I,J,K)
      elseif (Phase=='V') then
         RelPerm = ZInterfVapVolSats(I,J,K)
         Dens    = ZInterfVapDens(I,J,K)
         Enth    = ZInterfVapEnth(I,J,K)
         Visc    = ZInterfVapVisc(I,J,K)
         CentralDiffDens = ZInterfVapDens0(I,J,K)
      end if

      PreFactor = InterfPermsZ(I,J,K)*RelPerm*Dens*Enth/Visc
      GravTerm  = Grav*CentralDiffDens
      Answer = -PreFactor*(PresGrad - GravTerm)
   end if
else
   write(LogFile,*) 'Specified face is out of bounds -InterfMassFlux2'
   write(LogFile,*) 'Node: ',I,J,K,'Face: ',Face
   stop
end if
end
