function InterfSaltFlux2(NodeNumber,Face,Phase,Tempr,Presr,Salin) &
                        result(Answer)

use CommonVars, only: Prec,GridIndMat,LogFile,XNodeWidths,ZNodeWidths, &
   Pres,Temp,Sali,InterfPermsX,InterfPermsZ,Grav,SaltChemDiff,         &
   XInterfPoros,ZInterfPoros,XInterfLiqDens,XInterfLiqSali,            &
   XInterfLiqVisc,XInterfLiqDens0,XInterfLiqVolSats,XInterfVapDens,    &
   XInterfVapSali,XInterfVapVisc,XInterfVapDens0,XInterfVapVolSats,    &
   ZInterfLiqDens,ZInterfLiqSali,ZInterfLiqVisc,ZInterfLiqDens0,       &
   ZInterfLiqVolSats,ZInterfVapDens,ZInterfVapSali,ZInterfVapVisc,     &
   ZInterfVapDens0,ZInterfVapVolSats
implicit none
real(selected_real_kind(Prec)) :: Tempr,Presr,Salin,Answer,PresGrad, &
   NodeDist,PreFactor,RelPerm,Dens,Visc,InterfThermProp,GravTerm,    &
   CentralDiffDens,InterfDens,Saln,SaltGrad
integer :: NodeNumber,I,J,K,NodeExist,ChkNeighborExist
character :: Face,Phase

NodeExist = ChkNeighborExist(NodeNumber,Face)

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

if (NodeExist==1) then
   if (Face=='W') then
      NodeDist = XNodeWidths(I)/2.D0 + &
                 XNodeWidths(I-1)/2.D0
      PresGrad = (Presr-Pres(I-1,J,K))/NodeDist
      SaltGrad = (Salin-Sali(I-1,J,K))/NodeDist

      if (Phase=='L') then
         RelPerm = XInterfLiqVolSats(I-1,J,K)
         Dens    = XInterfLiqDens(I-1,J,K)
         Saln    = XInterfLiqSali(I-1,J,K)
         Visc    = XInterfLiqVisc(I-1,J,K)
      elseif (Phase=='V') then
         RelPerm = XInterfVapVolSats(I-1,J,K)
         Dens    = XInterfVapDens(I-1,J,K)
         Saln    = XInterfVapSali(I-1,J,K)
         Visc    = XInterfVapVisc(I-1,J,K)
      end if

      PreFactor = InterfPermsX(I-1,J,K)*RelPerm*Dens*Saln/Visc
      ! the 0.5 is because 1/2 the diffusive flux is added
      ! for each phase
      Answer = -PreFactor*PresGrad
   elseif (Face=='E') then
      NodeDist = XNodeWidths(I)/2.D0 + &
                 XNodeWidths(I+1)/2.D0
      PresGrad = (Pres(I+1,J,K)-Presr)/NodeDist
      SaltGrad = (Sali(I+1,J,K)-Salin)/NodeDist

      if (Phase=='L') then
         RelPerm = XInterfLiqVolSats(I,J,K)
         Dens    = XInterfLiqDens(I,J,K)
         Saln    = XInterfLiqSali(I,J,K)
         Visc    = XInterfLiqVisc(I,J,K)
      elseif (Phase=='V') then
         RelPerm = XInterfVapVolSats(I,J,K)
         Dens    = XInterfVapDens(I,J,K)
         Saln    = XInterfVapSali(I,J,K)
         Visc    = XInterfVapVisc(I,J,K)
      end if

      PreFactor = InterfPermsX(I,J,K)*RelPerm*Dens*Saln/Visc
      Answer = -PreFactor*PresGrad
   elseif (Face=='T') then
      NodeDist = ZNodeWidths(K)/2.D0 + &
                 ZNodeWidths(K-1)/2.D0
      PresGrad = (Presr-Pres(I,J,K-1))/NodeDist
      SaltGrad = (Salin-Sali(I,J,K-1))/NodeDist

      if (Phase=='L') then
         RelPerm = ZInterfLiqVolSats(I,J,K-1)
         Dens    = ZInterfLiqDens(I,J,K-1)
         Saln    = ZInterfLiqSali(I,J,K-1)
         Visc    = ZInterfLiqVisc(I,J,K-1)
         CentralDiffDens = ZInterfLiqDens0(I,J,K-1)
      elseif (Phase=='V') then
         RelPerm = ZInterfVapVolSats(I,J,K-1)
         Dens    = ZInterfVapDens(I,J,K-1)
         Saln    = ZInterfVapSali(I,J,K-1)
         Visc    = ZInterfVapVisc(I,J,K-1)
         CentralDiffDens = ZInterfVapDens0(I,J,K-1)
      end if

      PreFactor = InterfPermsZ(I,J,K-1)*RelPerm*Dens*Saln/Visc
      GravTerm  = Grav*CentralDiffDens
      Answer = -PreFactor*(PresGrad - GravTerm)
   elseif (Face=='B') then
      NodeDist = ZNodeWidths(K)/2.D0 + &
                 ZNodeWidths(K+1)/2.D0
      PresGrad = (Pres(I,J,K+1)-Presr)/NodeDist
      SaltGrad = (Sali(I,J,K+1)-Salin)/NodeDist

      if (Phase=='L') then
         RelPerm = ZInterfLiqVolSats(I,J,K)
         Dens    = ZInterfLiqDens(I,J,K)
         Saln    = ZInterfLiqSali(I,J,K)
         Visc    = ZInterfLiqVisc(I,J,K)
         CentralDiffDens = ZInterfLiqDens0(I,J,K)
      elseif (Phase=='V') then
         RelPerm = ZInterfVapVolSats(I,J,K)
         Dens    = ZInterfVapDens(I,J,K)
         Saln    = ZInterfVapSali(I,J,K)
         Visc    = ZInterfVapVisc(I,J,K)
         CentralDiffDens = ZInterfVapDens0(I,J,K)
      end if

      PreFactor = InterfPermsZ(I,J,K)*RelPerm*Dens*Saln/Visc
      GravTerm  = Grav*CentralDiffDens
      Answer = -PreFactor*(PresGrad - GravTerm)
   end if
else
   write(LogFile,*) 'Specified face is out of bounds -InterfMassFlux2'
   write(LogFile,*) 'Node: ',I,J,K,'Face: ',Face
   stop
end if
end
