function InterfThermProp(NodeNumber,Face,Phase,Prop, &
                         Tempr,Presr,Salin) result(Answer)
! Purpose: calculate the liquid or vapor density, enthalpy, 
!          salinity, volume saturation, or viscosity on a control 
!          volume interface, with upwinding 
! Input: NodeNumber, the number of the node whose interface value is
!        to be obtained; Face = 'W','E','T','B' indicating which face;
!        and Phase = 'L','V' indicating liquid or vapor density
!        Prop = 'D','E','S','V','C' for Density, Enthalpy, 
!        Salinity, Volume fraction, and visCosity
!        Tempr,Presr,Salin are the temperature(C), pressure (Pa), and
!        salinity (wt.fraction NaCl) at node NodeNumber

use CommonVars, only: Prec,GridIndMat,Temp,Pres,Sali,LogFile,   &
   XNodeWidths,ZNodeWidths,Grav,XInterfLiqDens,ZInterfLiqDens,  &
   XInterfVapDens,ZInterfVapDens,XInterfLiqEnth,ZInterfLiqEnth, &
   XInterfVapEnth,ZInterfVapEnth,XInterfLiqVolSats,             &
   ZInterfLiqVolSats,XInterfVapVolSats,ZInterfVapVolSats,       &
   XInterfLiqSali,ZInterfLiqSali,XInterfVapSali,ZInterfVapSali, &
   XInterfLiqVisc,ZInterfLiqVisc,XInterfVapVisc,ZInterfVapVisc
implicit none
real(selected_real_kind(Prec)) :: Answer,Tempr,Presr,Salin,NodeDist, &
   Val1,Val2,ThermProp,FlowDirection,PresGrad,CentralDiffDens, &
   InterfDens
integer :: NodeExist,NodeNumber,I,J,K,ChkNeighborExist
character :: Face,Phase,Prop

NodeExist = ChkNeighborExist(NodeNumber,Face)

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

if (NodeExist==1) then
   if (Face=='W') then
      if (Phase=='L') then
         if (Prop=='D') then
            Answer = XInterfLiqDens(I-1,J,K)
         elseif (Prop=='E') then
            Answer = XInterfLiqEnth(I-1,J,K)
         elseif (Prop=='S') then
            Answer = XInterfLiqSali(I-1,J,K)
         elseif (Prop=='V') then
            Answer = XInterfLiqVolSats(I-1,J,K)
         elseif (Prop=='C') then
            Answer = XInterfLiqVisc(I-1,J,K)
         end if
      elseif (Phase=='V') then
         if (Prop=='D') then
            Answer = XInterfVapDens(I-1,J,K)
         elseif (Prop=='E') then
            Answer = XInterfVapEnth(I-1,J,K)
         elseif (Prop=='S') then
            Answer = XInterfVapSali(I-1,J,K)
         elseif (Prop=='V') then
            Answer = XInterfVapVolSats(I-1,J,K)
         elseif (Prop=='C') then
            Answer = XInterfVapVisc(I-1,J,K)
         end if
      end if
   elseif (Face=='E') then
      if (Phase=='L') then
         if (Prop=='D') then
            Answer = XInterfLiqDens(I,J,K)
         elseif (Prop=='E') then
            Answer = XInterfLiqEnth(I,J,K)
         elseif (Prop=='S') then
            Answer = XInterfLiqSali(I,J,K)
         elseif (Prop=='V') then
            Answer = XInterfLiqVolSats(I,J,K)
         elseif (Prop=='C') then
            Answer = XInterfLiqVisc(I,J,K)
         end if
      elseif (Phase=='V') then
         if (Prop=='D') then
            Answer = XInterfVapDens(I,J,K)
         elseif (Prop=='E') then
            Answer = XInterfVapEnth(I,J,K)
         elseif (Prop=='S') then
            Answer = XInterfVapSali(I,J,K)
         elseif (Prop=='V') then
            Answer = XInterfVapVolSats(I,J,K)
         elseif (Prop=='C') then
            Answer = XInterfVapVisc(I,J,K)
         end if
      end if
   elseif (Face=='T') then
      if (Phase=='L') then
         if (Prop=='D') then
            Answer = ZInterfLiqDens(I,J,K-1)
         elseif (Prop=='E') then
            Answer = ZInterfLiqEnth(I,J,K-1)
         elseif (Prop=='S') then
            Answer = ZInterfLiqSali(I,J,K-1)
         elseif (Prop=='V') then
            Answer = ZInterfLiqVolSats(I,J,K-1)
         elseif (Prop=='C') then
            Answer = ZInterfLiqVisc(I,J,K-1)
         end if
      elseif (Phase=='V') then
         if (Prop=='D') then
            Answer = ZInterfVapDens(I,J,K-1)
         elseif (Prop=='E') then
            Answer = ZInterfVapEnth(I,J,K-1)
         elseif (Prop=='S') then
            Answer = ZInterfVapSali(I,J,K-1)
         elseif (Prop=='V') then
            Answer = ZInterfVapVolSats(I,J,K-1)
         elseif (Prop=='C') then
            Answer = ZInterfVapVisc(I,J,K-1)
         end if
      end if
   elseif (Face=='B') then
      if (Phase=='L') then
         if (Prop=='D') then
            Answer = ZInterfLiqDens(I,J,K)
         elseif (Prop=='E') then
            Answer = ZInterfLiqEnth(I,J,K)
         elseif (Prop=='S') then
            Answer = ZInterfLiqSali(I,J,K)
         elseif (Prop=='V') then
            Answer = ZInterfLiqVolSats(I,J,K)
         elseif (Prop=='C') then
            Answer = ZInterfLiqVisc(I,J,K)
         end if
      elseif (Phase=='V') then
         if (Prop=='D') then
            Answer = ZInterfVapDens(I,J,K)
         elseif (Prop=='E') then
            Answer = ZInterfVapEnth(I,J,K)
         elseif (Prop=='S') then
            Answer = ZInterfVapSali(I,J,K)
         elseif (Prop=='V') then
            Answer = ZInterfVapVolSats(I,J,K)
         elseif (Prop=='C') then
            Answer = ZInterfVapVisc(I,J,K)
         end if
      end if
   end if
else
   write(LogFile,*) 'Specified face is out of bounds -InterfThermProp'
   write(LogFile,*) 'Node: ',I,J,K,'Face: ',Face
   stop
end if
end
