subroutine LinearCoeffs
! compute linearization coefficients for the bulk density
! bulk enthalpy, to make these quantities linear functions of the
! temperature and pressure. DensCoeff1 and EnthCoeff1 are the
! slopes wrt temperature; DensCoeff2 and EnthCoeff2 are the slopes
! wrt pressure.

use CommonVars, only: Prec,OldPres,OldTemp,DensCoeff1,DensCoeff2, &
   EnthCoeff1,EnthCoeff2,NumNodesX,NumNodesY,NumNodesZ,OldSali
implicit none
real(selected_real_kind(Prec)) :: CalcBulkDens,CalcBulkEnth,      &
   DeltaTemp,DeltaPres,Dens1,Dens2,Enth1,Enth2
integer :: I,J,K

DeltaTemp = 1.D-3
DeltaPres = 1.D-3
do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ
         Dens1 = CalcBulkDens(OldTemp(I,J,K)-DeltaTemp, &
                              OldPres(I,J,K),OldSali(I,J,K))
         Dens2 = CalcBulkDens(OldTemp(I,J,K)+DeltaTemp, &
                              OldPres(I,J,K),OldSali(I,J,K))
         DensCoeff1(I,J,K) = (Dens2-Dens1)/(2.D0*DeltaTemp)

         Dens1 = CalcBulkDens(OldTemp(I,J,K), &
                              OldPres(I,J,K)-DeltaPres,OldSali(I,J,K))
         Dens2 = CalcBulkDens(OldTemp(I,J,K), &
                              OldPres(I,J,K)+DeltaPres,OldSali(I,J,K))
         DensCoeff2(I,J,K) = (Dens2-Dens1)/(2.D0*DeltaPres)

         Enth1 = CalcBulkEnth(OldTemp(I,J,K)-DeltaTemp, &
                              OldPres(I,J,K),OldSali(I,J,K))
         Enth2 = CalcBulkEnth(OldTemp(I,J,K)+DeltaTemp, &
                              OldPres(I,J,K),OldSali(I,J,K))
         EnthCoeff1(I,J,K) = (Enth2-Enth1)/(2.D0*DeltaTemp)

         Enth1 = CalcBulkEnth(OldTemp(I,J,K), &
                              OldPres(I,J,K)-DeltaPres,OldSali(I,J,K))
         Enth2 = CalcBulkEnth(OldTemp(I,J,K), &
                              OldPres(I,J,K)+DeltaPres,OldSali(I,J,K))
         EnthCoeff2(I,J,K) = (Enth2-Enth1)/(2.D0*DeltaPres)
      end do
   end do
end do
end
