function MassAccum(NodeNumber,Tempr,Presr,Salin,Linear) result(Answer)
! purpose: calculate the term of which the time derivative is
!          taken in the mass balance equation (phi*rho)
! input: NodeNumber = node at which to calculate this value
!        Tempr = temperature(C); Presr = pressure (Pa);
!        Salin = salinity(wt.fraction NaCl)
!        Linear = 'Y' if the linearized bulk density is to be used,
!               = 'N' otherwise

use CommonVars, only: Prec,GridIndMat,LogFile
implicit none
real(selected_real_kind(Prec)) :: Tempr,Presr,Salin,Answer,CalcPoros, &
   CalcLinrDens,CalcBulkDens
integer :: NodeNumber,I,J,K
character :: Linear

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

if (Linear=='Y') then
   Answer = CalcPoros(NodeNumber,Tempr,Presr,Salin)* &
            CalcLinrDens(I,J,K,Tempr,Presr)
elseif (Linear=='N') then
   Answer = CalcPoros(NodeNumber,Tempr,Presr,Salin)* &
            CalcBulkDens(Tempr,Presr,Salin)
else
   write(LogFile,*) 'Variable Linear not set - MassAccum.f90'
   write(LogFile,*) 'Terminating.'
   stop
end if
end
