subroutine NewtonIter(mumps_par)
! purpose: iterate the Newton-Raphson solutions until the
!          L2 norm of the residual drops below a certain
!          threshold

use CommonVars, only: Prec,Residual,LogFile,TimeStepFlag,Time,   &
   Delta_t,MaxSaltResid,MaxMassResid,MaxEnergyResid,             &
   MaxSaltResidPos,MaxEnergyResidPos,MaxMassResidPos,            &
   NumConverges,NewtonTolP,NewtonTolT,TotalTime,NewtonTolX
implicit none
include 'mpif.h'
include 'dmumps_struc.h'
integer :: Converged,IterNum,MaxIter,IntArray(1),IntArraySize,   &
   IERR,Root,I
type (DMUMPS_STRUC) :: mumps_par
real(selected_real_kind(Prec)) :: GlobalError

Converged=0
IterNum = 1
MaxIter = 30
IntArraySize = 1
Root = 0
do while ((Converged==0).and.(IterNum<=MaxIter))
   call Update

   call SolveSalinity

   call SolveSystem(1,3,IterNum,mumps_par)
   call ApplyCorrections

   call CalcMaxNorms

   if (((MaxMassResid<NewtonTolP).and.    &
        (MaxEnergyResid<NewtonTolT)).and. &
        (MaxSaltResid<NewtonTolX)) then
      Converged = 1
      if ((Time+Delta_t)>TotalTime) then
         IntArray(1) = 1
      else
         IntArray(1) = 0
      end if
   else
      IterNum = IterNum + 1
      IntArray(1) = 0
   end if
   do I=1,mumps_par%NSLAVES-1
      call MPI_SEND(IntArray,IntArraySize,MPI_INT,I,0, &
                    MPI_COMM_WORLD,IERR)
   end do
end do
if (Converged==1) then
   NumConverges = NumConverges + 1
   write(LogFile,*) 'Newton-Raphson iterations converged after '
   write(LogFile,*) IterNum,' iterations'
   write(LogFile,*) 'Maximum mass imbalance (kg/(m^3*yr)): ',          &
                     MaxMassResid
   write(LogFile,*) 'at node: ',MaxMassResidPos(1),MaxMassResidPos(2), &
                                MaxMassResidPos(3)
   write(LogFile,*) 'Maximum salt imbalance (kg/(m^3*yr)): ',          &
                     MaxSaltResid
   write(LogFile,*) 'at node: ',MaxSaltResidPos(1),MaxSaltResidPos(2), &
                                MaxSaltResidPos(3)
   write(LogFile,*) 'Maximum energy imbalance (kJ/(m^3*yr)): ',         &
                     MaxEnergyResid
   write(LogFile,*) 'at node: ',MaxEnergyResidPos(1),MaxEnergyResidPos(2), &
                                MaxEnergyResidPos(3)
   write(LogFile,*) 'current time(yr): ',Time/(86400.D0*365.D0)
   write(LogFile,*) 'time step size(days): ',Delta_t/86400.D0
   write(LogFile,*) '------------------------------------------'
else
   write(LogFile,*) 'Newton-Raphson iterations failed to converge.'
   write(LogFile,*) 'Final max residuals would have been'
   write(LogFile,*) 'Mass: ',MaxMassResid
   write(LogFile,*) 'Salt: ',MaxSaltResid
   write(LogFile,*) 'Energy: ',MaxEnergyResid
   NumConverges = 0
   TimeStepFlag = 1
end if
end
