subroutine NewtonIterStress
! purpose: iterate the Newton-Raphson solutions until the
!          L2 norm of the residual drops below a certain
!          threshold

use CommonVars, only: Prec,StressResidual,LogFile,TimeStepFlag,  &
   Time,Delta_t
implicit none
real(selected_real_kind(Prec)) :: L2NormX,NewtonTolX,NewtonTolZ, &
   L2NormZ
integer :: Converged,IterNum,MaxIter

NewtonTolX = 1.D-1
NewtonTolZ = 1.D-1
Converged=0
IterNum = 1
MaxIter = 5
do while ((Converged==0).and.(IterNum<=MaxIter))

   call CornerDisp
   call SolveSystemStress(1)
   call ApplyStressCorrect
   call CalcL2NormStress(L2NormX,L2NormZ)
   !write(*,*) L2NormX,L2NormZ
   !stop

   if ((L2NormX<NewtonTolX).and.(L2NormZ<NewtonTolZ)) then
      Converged = 1
   else
      IterNum = IterNum + 1
   end if
end do
if (Converged==1) then
   write(LogFile,*) '------------------------------------------------'
   write(LogFile,*) 'NR iterations for stress converged after'
   write(LogFile,*) IterNum,' iterations'
   write(LogFile,*) 'L2 norm of x disp residual (kg/m3): ',L2NormX
   write(LogFile,*) 'L2 norm of z disp residual (kJ/m3): ',L2NormZ
   write(LogFile,*) '------------------------------------------------'
else
   write(LogFile,*) 'NR iterations for stress failed to converge'
   TimeStepFlag = 1
   stop
end if
end
