subroutine PlaneFit(CoordMat,x,y,phi)
! Given the x and y coords for 4 points, together with the values
! of some function phi at those points, return the value of phi
! at a given fourth point. The fourth value is determined by the 
! best fitting plane through (x1,y1,phi1),(x2,y2,phi2),(x3,y3,phi3), 
! and (x4,y4,phi4) in (x,y,phi) space

!  2---------3
!  -         -
!  -         -
!  -         -
!  -         -
!  1---------4

use CommonVars, only: Prec
implicit none
real(selected_real_kind(Prec)) :: CoordMat(4,3),InvCoordMat(3,4), &
   Normal(3,1),b(4,1),L,x,y,phi,N,phi1,phi2,phi3,phi4
! CoordMat(1,:) = (x1,y1,phi1), CoordMat(2,:) = (x2,y2,phi2), etc...

! check whether the values of phi at the corners are all the same
phi1 = CoordMat(1,3)
phi2 = CoordMat(2,3)
phi3 = CoordMat(3,3)
phi4 = CoordMat(4,3)
if (((phi1 == phi2).and.(phi1 == phi3)).and. &
   (phi1 == phi4)) then
   phi = phi1
else  
   b(1,1) = 1.0
   b(2,1) = 1.0
   b(3,1) = 1.0
   b(4,1) = 1.0

   call PseudoInv(CoordMat,InvCoordMat)
   call MatrixMult(InvCoordMat,3,4,b,4,1,Normal)

   ! Give the normal vector unit length
   L = sqrt(Normal(1,1)**2.0 + Normal(2,1)**2.0 + Normal(3,1)**2.0)
   Normal(1,1) = Normal(1,1)/L
   Normal(2,1) = Normal(2,1)/L
   Normal(3,1) = Normal(3,1)/L

   ! for testing purposes
   !write(*,*) 'unit normal to plane: '
   !write(*,*) Normal(1,1),Normal(2,1),Normal(3,1)

   ! find the value of phi at x,y
   N = CoordMat(1,1)*Normal(1,1)+CoordMat(1,2)*Normal(2,1)+ &
      CoordMat(1,3)*Normal(3,1)
   phi = (N - (x*Normal(1,1) + y*Normal(2,1)))/Normal(3,1)
   if ((phi>=0.D0).or.(phi<0.D0)) then
   else
      phi = 0.D0
   end if
end if
end
