function PresResid(NodeNumber,Linear)  result(Answer)
! purpose: calculate the local mass balance residual
! input: NodeNumber = node at which to calculate the residual
! Linear = 'Y' if the linearized bulk density is the be used;
!        = 'N' otherwise

use CommonVars, only: Prec,Delta_t,OldPres,OldSali,OldTemp, &
   GridIndMat,XNodeWidths,ZNodeWidths,NumNodesX,NumNodesZ,  &
   PresGamma1X,PresGamma1Z,PresGamma2Z,Pres,PresNodeTypes,  &
   PresBoundVals,Temp,Sali
implicit none
real(selected_real_kind(Prec)) :: Answer,MassAccum,CoeffW, &
   CoeffE,CoeffT,CoeffB,CoeffP,Accum,NodeDist,GravTermB,   &
   GravTermT,TermP,TermW,TermE,TermT,TermB
character :: Linear
integer :: NodeNumber,I,J,K

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

Accum =  MassAccum(NodeNumber,Temp(I,J,K),Pres(I,J,K), &
         Sali(I,J,K),Linear)/Delta_t - &
         MassAccum(NodeNumber,OldTemp(I,J,K),OldPres(I,J,K), &
         OldSali(I,J,K),Linear)/Delta_t
if (I>1) then
   NodeDist = (XNodeWidths(I-1)+XNodeWidths(I))/2.D0
   CoeffW = PresGamma1X(I-1,J,K)/(NodeDist*XNodeWidths(I))
   TermW = -CoeffW*Pres(I-1,J,K)
else
   CoeffW = 0.0
   TermW  = 0.0
end if
if (I<NumNodesX) then
   NodeDist = (XNodeWidths(I)+XNodeWidths(I+1))/2.D0
   CoeffE = PresGamma1X(I,J,K)/(NodeDist*XNodeWidths(I))
   TermE = -CoeffE*Pres(I+1,J,K)
else
   CoeffE = 0.0
   TermE  = 0.0
end if
if (K>1) then
   NodeDist = (ZNodeWidths(K-1)+ZNodeWidths(K))/2.D0
   CoeffT = PresGamma1Z(I,J,K-1)/(NodeDist*ZNodeWidths(K))
   TermT = -CoeffT*Pres(I,J,K-1)
   GravTermT = -PresGamma2Z(I,J,K-1)/ZNodeWidths(K)
else
   CoeffT = 0.0
   TermT  = 0.0
   GravTermT = 0.0
end if
if (K<NumNodesZ) then
   NodeDist = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
   CoeffB = PresGamma1Z(I,J,K)/(NodeDist*ZNodeWidths(K))
   TermB = -CoeffB*Pres(I,J,K+1)
   GravTermB = PresGamma2Z(I,J,K)/ZNodeWidths(K)
else
   CoeffB = 0.0
   TermB  = 0.0
   GravTermB = 0.0
end if
CoeffP = CoeffW + CoeffE + CoeffB + CoeffT
TermP = CoeffP*Pres(I,J,K)
Answer = Accum + TermP + TermW + TermE + TermB + TermT + &
         GravTermB + GravTermT
if (PresNodeTypes(I,J,K)=='F') then
   if (NumNodesZ>1) then
      if ((K==1).or.(K==NumNodesZ)) then
         Answer = Answer - PresBoundVals(I,J,K)/ZNodeWidths(K)
      else
         Answer = Answer - PresBoundVals(I,J,K)/XNodeWidths(I)
      end if
   else
      Answer = Answer - PresBoundVals(I,J,K)/XNodeWidths(I)
   end if
elseif (PresNodeTypes(I,J,K)=='C') then
   Answer = 0.0
end if
Answer = Answer*3.2D7
end
