function SaliResidDeriv(Node1,Node2,Primary,Linear) result(Answer)
! purpose: calculate the derivative of the salt balance residual
!          at node Node1 with respect to an increase in the
!          temperature, pressure, or salinity at node Node2
! input: Node1 = node at which residual is calculated
!        Node2 = node at which a variable is being increased
!        Primary = 'T','P','X', which variable to increase
!        Linear = 'Y' if the linearized bulk density is to be used;
!               = 'N' otherwise

use CommonVars, only: Prec,GridIndMat,Pres,Temp,Sali,DeltaT,DeltaT, &
   DeltaT,SaliGamma1X,SaliGamma1Z,NumNodesX,NumNodesZ,XNodeWidths,  &
   ZNodeWidths,Delta_t,XInterfLiqDiff,ZInterfLiqDiff,DeltaP,DeltaX, &
   XInterfVapDiff,XInterfLiqDiff,LiquidSali,VaporSali,LiquidDens,   &
   VaporDens,ZInterfVapDiff
implicit none
real(selected_real_kind(Prec)) :: Answer,Temp1,Temp2,Resid1,Resid2, &
   Pres1,Pres2,Sali1,Sali2,MassAccum,TermW,TermE,TermT,TermB,       &
   DAccumDT,Accum1,Accum2,NodeDist,DAccumDP,DAccumDX,DDiffuseDP,    &
   DDiffuseDT,DDiffuseDX,Diffuse1,Diffuse2,SaltDiffuseTerm,Tempr,   &
   Presr,Salin,SaltAccum,CalcBulkDens,CalcPoros
character :: Primary,Direction,RelativeDirection,Linear
integer Node1,Node2,I,J,K,PhaseIndex,PhaseInd1,PhaseInd2,PhaseInd

I = GridIndMat(Node1,1)
J = GridIndMat(Node1,2)
K = GridIndMat(Node1,3)

Tempr = Temp(I,J,K)
Presr = Pres(I,J,K)
Salin = Sali(I,J,K)

Sali1 = Salin - DeltaX
Sali2 = Salin + DeltaX
PhaseInd1 = PhaseIndex(Tempr,Presr,Sali1)
PhaseInd2 = PhaseIndex(Tempr,Presr,Sali2)
if (PhaseInd1.ne.PhaseInd2) then
   Sali1 = Salin
end if
Accum1 = MassAccum(Node1,Tempr,Presr,Sali1,Linear)
Accum2 = MassAccum(Node1,Tempr,Presr,Sali2,Linear)
DAccumDX = Salin*(Accum2-Accum1)/(Sali2-Sali1) + &
           CalcPoros(Node1,Tempr,Presr,Salin)* &
           CalcBulkDens(Tempr,Presr,Salin)

end
