function SaltDiffuseTerm(Node1,Node2,Tempr,Presr,Salin) &
   result(Answer)
! Purpose: calculate the sum of the diffusive terms for the
!          salt balance equation
! input: Node1 = # of the node at which to calculate this
!        sum; Node2 = # of the node at which to apply the
!        input temperature (C), pressure (Pa), and salinity 
!        (wt.fraction NaCl) 

use CommonVars, only: Prec,GridIndMat,XInterfLiqDiff,XInterfVapDiff, &
   ZInterfLiqDiff,ZInterfVapDiff,Temp,Pres,Sali,LiquidSali,          &
   LiquidDens,VaporSali,VaporDens,XNodeWidths,ZNodeWidths,           &
   NumNodesX,NumNodesZ
implicit none
real(selected_real_kind(Prec)) :: Tempr,Presr,Salin,Answer,    &
   LiqCoeffW,LiqCoeffE,LiqCoeffT,LiqCoeffB,LiqCoeffP,          &
   VapCoeffW,VapCoeffE,VapCoeffT,VapCoeffB,VapCoeffP,NodeDist, &
   LiqRhoXP,LiqRhoXW,LiqRhoXE,LiqRhoXT,LiqRhoXB,VapRhoXP,      &
   VapRhoXW,VapRhoXE,VapRhoXT,VapRhoXB,Tempr2,Presr2,Salin2,   &
   S2PhaseL,S2PhaseV,D2PhaseL,D2PhaseV,LiqTermP,LiqTermW,      &
   LiqTermE,LiqTermT,LiqTermB,VapTermP,VapTermW,VapTermE,      &
   VapTermT,VapTermB,CalcBulkDens
integer :: Node1,Node2,I,J,K,PhaseInd,PhaseIndex
character :: RelativeDirection,Direction

I = GridIndMat(Node1,1)
J = GridIndMat(Node1,2)
K = GridIndMat(Node1,3)

if (Node1==Node2) then
   Direction = 'P'
else
   Direction = RelativeDirection(Node1,Node2)
end if

if (I>1) then
   NodeDist = (XNodeWidths(I-1)+XNodeWidths(I))/2.D0
   LiqCoeffW = XInterfLiqDiff(I-1,J,K)/(NodeDist*XNodeWidths(I))
   VapCoeffW = XInterfVapDiff(I-1,J,K)/(NodeDist*XNodeWidths(I))
   if (Direction.ne.'W') then
      Tempr2 = Temp(I-1,J,K)
      Presr2 = Pres(I-1,J,K)
      Salin2 = Sali(I-1,J,K)
   else
      Tempr2 = Tempr
      Presr2 = Presr
      Salin2 = Salin
   end if
   if (PhaseInd==0) then
      LiqRhoXW = D2PhaseL(Tempr2,Presr2)* &
                 S2PhaseL(Tempr2,Presr2)
      VapRhoXW = D2PhaseV(Tempr2,Presr2)* &
                 S2PhaseV(Tempr2,Presr2)
   elseif (PhaseInd<0) then
      LiqRhoXW = CalcBulkDens(Tempr2,Presr2,Salin2)*Salin2
      VapRhoXW = 0.D0
   elseif (PhaseInd>0) then
      LiqRhoXW = 0.D0
      VapRhoXW = CalcBulkDens(Tempr2,Presr2,Salin2)*Salin2
   end if
else
   LiqCoeffW = 0.D0
   VapCoeffW = 0.D0
   LiqRhoXW = 0.D0
   VapRhoXW = 0.D0
end if
if (I<NumNodesX) then
   NodeDist = (XNodeWidths(I)+XNodeWidths(I+1))/2.D0
   LiqCoeffE = XInterfLiqDiff(I,J,K)/(NodeDist*XNodeWidths(I))
   VapCoeffE = XInterfVapDiff(I,J,K)/(NodeDist*XNodeWidths(I))
   if (Direction.ne.'E') then
      Tempr2 = Temp(I+1,J,K)
      Presr2 = Pres(I+1,J,K)
      Salin2 = Sali(I+1,J,K)
   else
      Tempr2 = Tempr
      Presr2 = Presr
      Salin2 = Salin
   end if
   if (PhaseInd==0) then
      LiqRhoXE = D2PhaseL(Tempr2,Presr2)* &
                 S2PhaseL(Tempr2,Presr2)
      VapRhoXE = D2PhaseV(Tempr2,Presr2)* &
                 S2PhaseV(Tempr2,Presr2)
   elseif (PhaseInd<0) then
      LiqRhoXE = CalcBulkDens(Tempr2,Presr2,Salin2)*Salin2
      VapRhoXE = 0.D0
   elseif (PhaseInd>0) then
      LiqRhoXE = 0.D0
      VapRhoXE = CalcBulkDens(Tempr2,Presr2,Salin2)*Salin2
   end if
else
   LiqCoeffE = 0.D0
   VapCoeffE = 0.D0
   LiqRhoXE = 0.D0
   VapRhoXE = 0.D0
end if
if (K>1) then
   NodeDist = (ZNodeWidths(K-1)+ZNodeWidths(K))/2.D0
   LiqCoeffT = ZInterfLiqDiff(I,J,K-1)/(NodeDist*ZNodeWidths(K))
   VapCoeffT = ZInterfVapDiff(I,J,K-1)/(NodeDist*ZNodeWidths(K))
   if (Direction.ne.'T') then
      Tempr2 = Temp(I,J,K-1)
      Presr2 = Pres(I,J,K-1)
      Salin2 = Sali(I,J,K-1)
   else       
      Tempr2 = Tempr
      Presr2 = Presr
      Salin2 = Salin
   end if
   if (PhaseInd==0) then
      LiqRhoXT = D2PhaseL(Tempr2,Presr2)* &
                 S2PhaseL(Tempr2,Presr2)
      VapRhoXT = D2PhaseV(Tempr2,Presr2)* &
                 S2PhaseV(Tempr2,Presr2)
   elseif (PhaseInd<0) then
      LiqRhoXT = CalcBulkDens(Tempr2,Presr2,Salin2)*Salin2
      VapRhoXT = 0.D0
   elseif (PhaseInd>0) then
      LiqRhoXT = 0.D0
      VapRhoXT = CalcBulkDens(Tempr2,Presr2,Salin2)*Salin2
   end if
else
   LiqCoeffT = 0.D0
   VapCoeffT = 0.D0
   LiqRhoXT = 0.D0
   VapRhoXT = 0.D0
end if
if (K<NumNodesZ) then
   NodeDist = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
   LiqCoeffB = ZInterfLiqDiff(I,J,K)/(NodeDist*ZNodeWidths(K))
   VapCoeffB = ZInterfVapDiff(I,J,K)/(NodeDist*ZNodeWidths(K))
   if (Direction.ne.'B') then
      Tempr2 = Temp(I,J,K+1)
      Presr2 = Pres(I,J,K+1)
      Salin2 = Sali(I,J,K+1)
   else
      Tempr2 = Tempr
      Presr2 = Presr
      Salin2 = Salin
   end if
   if (PhaseInd==0) then
      LiqRhoXB = D2PhaseL(Tempr2,Presr2)* &
                 S2PhaseL(Tempr2,Presr2)
      VapRhoXB = D2PhaseV(Tempr2,Presr2)* &
                 S2PhaseV(Tempr2,Presr2)
   elseif (PhaseInd<0) then
      LiqRhoXB = CalcBulkDens(Tempr2,Presr2,Salin2)*Salin2
      VapRhoXB = 0.D0
   elseif (PhaseInd>0) then
      LiqRhoXB = 0.D0
      VapRhoXB = CalcBulkDens(Tempr2,Presr2,Salin2)*Salin2
   end if
else
   LiqCoeffB = 0.D0
   VapCoeffB = 0.D0
   LiqRhoXB = 0.D0
   VapRhoXB = 0.D0
end if
LiqTermW = -LiqCoeffW*LiqRhoXW
VapTermW = -VapCoeffW*VapRhoXW
LiqTermE = -LiqCoeffE*LiqRhoXE
VapTermE = -VapCoeffE*VapRhoXE
LiqTermT = -LiqCoeffT*LiqRhoXT
VapTermT = -VapCoeffT*VapRhoXT
LiqTermB = -LiqCoeffB*LiqRhoXB
VapTermB = -VapCoeffB*VapRhoXB

if (Direction.ne.'P') then
   Tempr2 = Temp(I,J,K)
   Presr2 = Pres(I,J,K)
   Salin2 = Sali(I,J,K)
else
   Tempr2 = Tempr
   Presr2 = Presr
   Salin2 = Salin
end if
if (PhaseInd==0) then
   LiqRhoXP = D2PhaseL(Tempr2,Presr2)* &
              S2PhaseL(Tempr2,Presr2)
   VapRhoXP = D2PhaseV(Tempr2,Presr2)* &
              S2PhaseV(Tempr2,Presr2)
elseif (PhaseInd<0) then
   LiqRhoXP = CalcBulkDens(Tempr2,Presr2,Salin2)*Salin2
   VapRhoXP = 0.D0
elseif (PhaseInd>0) then
   LiqRhoXP = 0.D0
   VapRhoXP = CalcBulkDens(Tempr2,Presr2,Salin2)*Salin2
end if

LiqTermP = (LiqCoeffW+LiqCoeffE+LiqCoeffT+LiqCoeffB)*LiqRhoXP
VapTermP = (VapCoeffW+VapCoeffE+VapCoeffT+VapCoeffB)*VapRhoXP

Answer = LiqTermP+LiqTermW+LiqTermE+LiqTermT+LiqTermB+ &
         VapTermP+VapTermW+VapTermE+VapTermT+VapTermB
Answer = 0.0
end
