subroutine SetNodeNumbers
! Purpose: to assign a number M to every node I,J,K; to specify
! connections between nodes

! NodeNumMat(I,J,K) = number of the node at grid position I,J,K
!
! GridIndMat(M,1) = I index of node number M
! GridIndMat(M,2) = J index of node number M
! GridIndMat(M,3) = K index of node number M
!
! ConnectMat(M,1) = node number of the west   neighbor of node M
! ConnectMat(M,2) = node number of the east   neighbor of node M
! ConnectMat(M,3) = node number of the top    neighbor of node M
! ConnectMat(M,4) = node number of the bottom neighbor of node M
! ConnectMat(M,5) = node number of the north  neighbor of node M
! ConnectMat(M,6) = node number of the south  neighbor of node M
! -1 will be stored when the neighboring node does not exist

use CommonVars, only: NodeNumMat,GridIndMat,ConnectMat,NumNodesX, &
   NumNodesY,NumNodesZ,NumNodesTotal
implicit none
integer I,J,K,NodeNumber,L,EastI,EastJ,EastK,WestI,WestJ,WestK, &
   TopI,TopJ,TopK,BotI,BotJ,BotK,NorthI,NorthJ,NorthK,          &
   SouthI,SouthJ,SouthK

do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ
         NodeNumber = K + (J-1)*NumNodesZ + (I-1)*NumNodesZ
         NodeNumMat(I,J,K) = NodeNumber
         GridIndMat(NodeNumber,1) = I
         GridIndMat(NodeNumber,2) = J
         GridIndMat(NodeNumber,3) = K
      end do
   end do
end do
NumNodesTotal = NodeNumber

! set up the connectivity matrix
do L=1,NumNodesTotal
   I = GridIndMat(L,1)
   J = GridIndMat(L,2)
   K = GridIndMat(L,3)

   WestI = I-1
   WestJ = J
   WestK = K
   if (WestI >= 1) then
      ConnectMat(L,1) = NodeNumMat(WestI,WestJ,WestK)
   else
      ConnectMat(L,1) = -1
   end if

   EastI = I+1
   EastJ = J
   EastK = K
   if (EastI <= NumNodesX) then
      ConnectMat(L,2) = NodeNumMat(EastI,EastJ,EastK)
   else
      ConnectMat(L,2) = -1
   end if

   TopI = I
   TopJ = J
   TopK = K-1
   if (TopK >= 1) then
      ConnectMat(L,3) = NodeNumMat(TopI,TopJ,TopK)
   else
      ConnectMat(L,3) = -1
   end if 

   BotI = I
   BotJ = J
   BotK = K+1
   if (BotK <= NumNodesZ) then
      ConnectMat(L,4) = NodeNumMat(BotI,BotJ,BotK)
   else
      ConnectMat(L,4) = -1
   end if

   NorthI = I
   NorthJ = J-1
   NorthK = K
   if (NorthJ >= 1) then
      ConnectMat(L,5) = NodeNumMat(NorthI,NorthJ,NorthK) 
   else
      ConnectMat(L,5) = -1
   end if

   SouthI = I
   SouthJ = J+1
   SouthK = K
   if (SouthJ <= NumNodesY) then
      ConnectMat(L,6) = NodeNumMat(SouthI,SouthJ,SouthK)
   else
      ConnectMat(L,6) = -1
   end if
end do
end
