subroutine SolveSystemStress(FormJacobian)
! purpose: form the Jacobian matrix; store it in sparse triplet
!          format; form the Residual vector; solve the system
!          J*Corrections = -Residual for the corrections to pressure
!          temperature, and salinity

! ConnectMat(M,1) = node number of the west   neighbor of node M
! ConnectMat(M,2) = node number of the east   neighbor of node M
! ConnectMat(M,3) = node number of the top    neighbor of node M
! ConnectMat(M,4) = node number of the bottom neighbor of node M
! ConnectMat(M,5) = node number of the north  neighbor of node M
! ConnectMat(M,6) = node number of the south  neighbor of node M
! -1 will be stored when the neighboring node does not exist
!
! Jacobian(N) is the Nth non-zero entry of the Jacobian matrix J(L,M)
! JacobianL(N) is the L index of the Nth nonzero entry
! JacobianM(N) is the M index of the Nth nonzero entry

use CommonVars, only: Prec,ConnectMat,StressJacobian,StressJacobianL, &
   StressJacobianM,NumNodesTotal,StressResidual,GridIndMat,DispX,     &
   DispZ,StressCorrections,StressNodeTypes
implicit none
real(selected_real_kind(Prec)) :: DispXResidDeriv,DispXResid, &
   MinusResid(2*NumNodesTotal),AbsTol,RelTol,DispZResidDeriv, &
   DispZResid
integer :: L,M,N,Counter,I,J,K,SystemOrder,NumNonZero,MaxInner, &
   MaxOuter,FormJacobian

Counter = 0
do L=1,NumNodesTotal
   I = GridIndMat(L,1)
   J = GridIndMat(L,2)
   K = GridIndMat(L,3)
   !------------------------------------------------------------------
   ! x displacement - main block
   !------------------------------------------------------------------
   if (FormJacobian==1) then
      Counter = Counter + 1
      StressJacobianL(Counter) = L
      StressJacobianM(Counter) = L
      if (StressNodeTypes(I,J,K).ne.'P') then
         StressJacobian(Counter) = DispXResidDeriv(L,L,'X')
      else
         StressJacobian(Counter) = 1.D0
      end if

      do N = 1,4
         M = ConnectMat(L,N)
         if (M.ne.-1) then
            Counter = Counter + 1
            StressJacobianL(Counter) = L
            StressJacobianM(Counter) = M
            !if (StressNodeTypes(I,J,K).ne.'P') then
            !   StressJacobian(Counter) = DispXResidDeriv(L,M,'X')
            !else
               StressJacobian(Counter) = 0.D0
            !end if
         end if
      end do
   end if

   StressResidual(L) = DispXResid(L)
   MinusResid(L) = -StressResidual(L)
   StressCorrections(L) = 1.D-10
   !------------------------------------------------------------------
   ! z displacement - main block
   !------------------------------------------------------------------
   if (FormJacobian==1) then
      Counter = Counter + 1
      StressJacobianL(Counter) = L + NumNodesTotal
      StressJacobianM(Counter) = L + NumNodesTotal
      if (StressNodeTypes(I,J,K).ne.'P') then
         StressJacobian(Counter) = DispZResidDeriv(L,L,'Z')
      else
         StressJacobian(Counter) = 1.D0
      end if

      do N = 1,4
         M = ConnectMat(L,N)
         if (M.ne.-1) then
            Counter = Counter + 1
            StressJacobianL(Counter) = L + NumNodesTotal
            StressJacobianM(Counter) = M + NumNodesTotal
            !if (StressNodeTypes(I,J,K).ne.'P') then
            !   StressJacobian(Counter) = DispZResidDeriv(L,M,'Z')
            !else
               StressJacobian(Counter) = 0.D0
            !end if
         end if
      end do
   end if

   StressResidual(L+NumNodesTotal) = DispZResid(L)
   MinusResid(L+NumNodesTotal) = -StressResidual(L+NumNodesTotal)
   StressCorrections(L+NumNodestotal) = 1.D-10
   !------------------------------------------------------------------
end do

! solve for the corrections to x and z displacements
! work up to SystemOrder = 2*NumNodesTotal
SystemOrder = 2*NumNodesTotal
NumNonZero  = Counter
MaxOuter = 100
MaxInner = MIN(MaxOuter-1,SystemOrder-1)
AbsTol = 1.D-20
RelTol = 1.D-20
call mgmres_st (SystemOrder,NumNonZero,StressJacobianL,StressJacobianM, &
                StressJacobian,StressCorrections,MinusResid,MaxOuter,   &
                MaxInner,AbsTol,RelTol)
end
