function TempResid(NodeNumber,Linear)  result(Answer)
! purpose: calculate the local mass balance residual
! input: NodeNumber = node at which to calculate the residual
!        Linear = 'Y' if the linearized bulk enthalpy and bulk
!                  density are to be used; 'N' otherwise

use CommonVars, only: Prec,Delta_t,OldPres,OldSali,OldTemp, &
   GridIndMat,XNodeWidths,ZNodeWidths,NumNodesX,NumNodesZ,  &
   TempGamma1X,TempGamma1Z,TempGamma2Z,Pres,TempNodeTypes,  &
   TempBoundVals,Temp,Sali,XInterfEffCond,ZInterfEffCond
implicit none
real(selected_real_kind(Prec)) :: Answer,EnergyAccum,PresCoeffW, &
   PresCoeffE,PresCoeffT,PresCoeffB,PresCoeffP,Accum,NodeDist,   &
   GravTempTermB,GravTempTermT,PresTermP,PresTermW,PresTermE,    &
   PresTermT,PresTermB,TempCoeffW,TempCoeffE,TempCoeffT,         &
   TempCoeffB,TempCoeffP,TempTermE,TempTermW,TempTermT,          &
   TempTermB,TempTermP
character :: Linear
integer :: NodeNumber,I,J,K

I = GridIndMat(NodeNumber,1)
J = GridIndMat(NodeNumber,2)
K = GridIndMat(NodeNumber,3)

Accum =  EnergyAccum(NodeNumber,Temp(I,J,K),Pres(I,J,K), &
         Sali(I,J,K),'B',Linear)/Delta_t - &
         EnergyAccum(NodeNumber,OldTemp(I,J,K),OldPres(I,J,K), &
         OldSali(I,J,K),'B',Linear)/Delta_t
if (I>1) then
   NodeDist = (XNodeWidths(I-1)+XNodeWidths(I))/2.D0
   PresCoeffW = TempGamma1X(I-1,J,K)/(NodeDist*XNodeWidths(I))
   TempCoeffW = XInterfEffCond(I-1,J,K)/(NodeDist*XNodeWidths(I))
   PresTermW = -PresCoeffW*Pres(I-1,J,K)
   TempTermW = -TempCoeffW*Temp(I-1,J,K)
else
   PresCoeffW = 0.0
   TempCoeffW = 0.0
   PresTermW  = 0.0
   TempTermW  = 0.0
end if
if (I<NumNodesX) then
   NodeDist = (XNodeWidths(I)+XNodeWidths(I+1))/2.D0
   PresCoeffE = TempGamma1X(I,J,K)/(NodeDist*XNodeWidths(I))
   TempCoeffE = XInterfEffCond(I,J,K)/(NodeDist*XNodeWidths(I))
   PresTermE = -PresCoeffE*Pres(I+1,J,K)
   TempTermE = -TempCoeffE*Temp(I+1,J,K)
else
   PresCoeffE = 0.0
   TempCoeffE = 0.0
   PresTermE  = 0.0
   TempTermE  = 0.0
end if
if (K>1) then
   NodeDist = (ZNodeWidths(K-1)+ZNodeWidths(K))/2.D0
   PresCoeffT = TempGamma1Z(I,J,K-1)/(NodeDist*ZNodeWidths(K))
   TempCoeffT = ZInterfEffCond(I,J,K-1)/(NodeDist*ZNodeWidths(K))
   PresTermT = -PresCoeffT*Pres(I,J,K-1)
   TempTermT = -TempCoeffT*Temp(I,J,K-1)
   GravTempTermT = -TempGamma2Z(I,J,K-1)/ZNodeWidths(K)
else
   PresCoeffT = 0.0
   TempCoeffT = 0.0
   PresTermT  = 0.0
   TempTermT  = 0.0
   GravTempTermT = 0.0
end if
if (K<NumNodesZ) then
   NodeDist = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
   PresCoeffB = TempGamma1Z(I,J,K)/(NodeDist*ZNodeWidths(K))
   TempCoeffB = ZInterfEffCond(I,J,K)/(NodeDist*ZNodeWidths(K))
   PresTermB = -PresCoeffB*Pres(I,J,K+1)
   TempTermB = -TempCoeffB*Temp(I,J,K+1)
   GravTempTermB = TempGamma2Z(I,J,K)/ZNodeWidths(K)
else
   PresCoeffB = 0.0
   TempCoeffB = 0.0
   PresTermB  = 0.0
   TempTermB  = 0.0
   GravTempTermB = 0.0
end if
PresCoeffP = PresCoeffW + PresCoeffE + PresCoeffB + PresCoeffT
TempCoeffP = TempCoeffW + TempCoeffE + TempCoeffB + TempCoeffT
PresTermP = PresCoeffP*Pres(I,J,K)
TempTermP = TempCoeffP*Temp(I,J,K)
Answer = Accum + PresTermP + PresTermW + PresTermE + PresTermB + &
         PresTermT + TempTermP + TempTermW + TempTermE + &
         TempTermB + TempTermT + GravTempTermB + GravTempTermT
if (TempNodeTypes(I,J,K)=='F') then
   if (NumNodesZ>1) then
      if ((K==1).or.(K==NumNodesZ)) then
         if (K==1) then
            Answer = Answer + TempBoundVals(I,J,K)/ZNodeWidths(K)
         elseif (K==NumNodesZ) then
            Answer = Answer - TempBoundVals(I,J,K)/ZNodeWidths(K)
         end if
      elseif ((I==1).or.(I==NumNodesX)) then
         if (I==1) then
            Answer = Answer + TempBoundVals(I,J,K)/XNodeWidths(I)
         elseif (I==NumNodesX) then
            Answer = Answer - TempBoundVals(I,J,K)/XNodeWidths(I)
         end if
      end if
   else
      if (I==1) then
         Answer = Answer + TempBoundVals(I,J,K)/XNodeWidths(I)
      elseif (I==NumNodesX) then
         Answer = Answer - TempBoundVals(I,J,K)/XNodeWidths(I)
      end if
   end if
elseif ((TempNodeTypes(I,J,K)=='C').or. &
        (TempNodeTypes(I,J,K)=='U')) then
   Answer = 0.0
end if
Answer = Answer*3.2D4
end
