function TempResidDeriv(Node1,Node2,Primary,Linear) result(Answer)
! purpose: calculate the derivative of the energy balance residual
!          at node Node1 with respect to an increase in the
!          temperature, pressure, or salinity at node Node2
! input: Node1 = node at which residual is calculated
!        Node2 = node at which a variable is being increased
!        Primary = 'T','P','X', which variable to increase
!        Linear = 'Y' if the linearized bulk enthalpy and bulk
!                  density are to be used; 'N' otherwise

use CommonVars, only: Prec,GridIndMat,Pres,Temp,Sali,DeltaT, &
   TempGamma1X,TempGamma1Z,NumNodesX,NumNodesZ,XNodeWidths,  &
   ZNodeWidths,Delta_t,XInterfEffCond,ZInterfEffCond,DeltaP, &
   DeltaX
implicit none
real(selected_real_kind(Prec)) :: Answer,Temp1,Temp2,Resid1,Resid2, &
   Pres1,Pres2,Sali1,Sali2,EnergyAccum,TermW,TermE,TermT,TermB,     &
   DAccumDT,Accum1,Accum2,NodeDist,DAccumDP,DAccumDX
character :: Primary,Direction,RelativeDirection,Linear
integer Node1,Node2,I,J,K,PhaseIndex,PhaseInd1,PhaseInd2

I = GridIndMat(Node1,1)
J = GridIndMat(Node1,2)
K = GridIndMat(Node1,3)

if (Node1==Node2) then
   if (Primary=='P') then
      Pres1 = Pres(I,J,K) - DeltaP
      Pres2 = Pres(I,J,K) + DeltaP
      PhaseInd1 = PhaseIndex(Temp(I,J,K),Pres1,Sali(I,J,K))
      PhaseInd2 = PhaseIndex(Temp(I,J,K),Pres2,Sali(I,J,K))
      if (PhaseInd1.ne.PhaseInd2) then
         Pres1 = Pres(I,J,K)
      end if
      Accum1 = EnergyAccum(Node1,Temp(I,J,K),Pres1,Sali(I,J,K),'B',Linear)
      Accum2 = EnergyAccum(Node1,Temp(I,J,K),Pres2,Sali(I,J,K),'B',Linear)
      DAccumDP = (Accum2-Accum1)/(Pres2-Pres1)
      if (I<NumNodesX) then
         NodeDist = (XNodeWidths(I)+XNodeWidths(I+1))/2.D0
         TermE = TempGamma1X(I,J,K)/(NodeDist*XNodeWidths(I))
      else
         TermE = 0.D0
      end if
      if (I>1) then
         NodeDist = (XNodeWidths(I-1)+XNodeWidths(I))/2.D0
         TermW = TempGamma1X(I-1,J,K)/(NodeDist*XNodeWidths(I))
      else
         TermW = 0.D0
      end if
      if (K<NumNodesZ) then
         NodeDist = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
         TermB = TempGamma1Z(I,J,K)/(NodeDist*ZNodeWidths(K))
      else
         TermB = 0.D0
      end if
      if (K>1) then
         NodeDist = (ZNodeWidths(K-1)+ZNodeWidths(K))/2.D0
         TermT = TempGamma1Z(I,J,K-1)/(NodeDist*ZNodeWidths(K))
      else
         TermT = 0.D0
      end if
      Answer = DAccumDP/Delta_t + TermW + TermE + TermT + TermB
   elseif (Primary=='T') then
      Temp1 = Temp(I,J,K) - DeltaT
      Temp2 = Temp(I,J,K) + DeltaT
      PhaseInd1 = PhaseIndex(Temp1,Pres(I,J,K),Sali(I,J,K))
      PhaseInd2 = PhaseIndex(Temp2,Pres(I,J,K),Sali(I,J,K))
      if (PhaseInd1.ne.PhaseInd2) then
         Temp1 = Temp(I,J,K)
      end if
      Accum1 = EnergyAccum(Node1,Temp1,Pres(I,J,K),Sali(I,J,K),'B',Linear)
      Accum2 = EnergyAccum(Node1,Temp2,Pres(I,J,K),Sali(I,J,K),'B',Linear)
      DAccumDT = (Accum2-Accum1)/(Temp2-Temp1)
      if (I<NumNodesX) then
         NodeDist = (XNodeWidths(I)+XNodeWidths(I+1))/2.D0
         TermE = XInterfEffCond(I,J,K)/(NodeDist*XNodeWidths(I))
      else
         TermE = 0.D0
      end if
      if (I>1) then
         NodeDist = (XNodeWidths(I-1)+XNodeWidths(I))/2.D0
         TermW = XInterfEffCond(I-1,J,K)/(NodeDist*XNodeWidths(I))
      else
         TermW = 0.D0
      end if
      if (K<NumNodesZ) then
         NodeDist = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
         TermB = ZInterfEffCond(I,J,K)/(NodeDist*ZNodeWidths(K))
      else
         TermB = 0.D0
      end if
      if (K>1) then
         NodeDist = (ZNodeWidths(K-1)+ZNodeWidths(K))/2.D0
         TermT = ZInterfEffCond(I,J,K-1)/(NodeDist*ZNodeWidths(K))
      else
         TermT = 0.D0
      end if
      Answer = DAccumDT/Delta_t + TermW + TermE + TermT + TermB
   elseif (Primary=='X') then
      Sali1 = Sali(I,J,K) - DeltaX
      Sali2 = Sali(I,J,K) + DeltaX
      PhaseInd1 = PhaseIndex(Temp(I,J,K),Pres(I,J,K),Sali1)
      PhaseInd2 = PhaseIndex(Temp(I,J,K),Pres(I,J,K),Sali2)
      if (PhaseInd1.ne.PhaseInd2) then
         Sali1 = Sali(I,J,K)
      end if
      Accum1 = EnergyAccum(Node1,Temp(I,J,K),Pres(I,J,K),Sali1,'B',Linear)
      Accum2 = EnergyAccum(Node1,Temp(I,J,K),Pres(I,J,K),Sali2,'B',Linear)
      DAccumDX = (Accum2-Accum1)/(Sali2-Sali1)
      Answer = DAccumDX/Delta_t
   end if
else
   Direction = RelativeDirection(Node1,Node2)
   if (Direction=='W') then
      if (Primary=='P') then
         NodeDist = (XNodeWidths(I-1)+XNodeWidths(I))/2.D0
         Answer = -TempGamma1X(I-1,J,K)/(NodeDist*XNodeWidths(I))
      elseif (Primary=='T') then
         NodeDist = (XNodeWidths(I-1)+XNodeWidths(I))/2.D0
         Answer = -XInterfEffCond(I-1,J,K)/(NodeDist*XNodeWidths(I))
      elseif (Primary=='X') then
         Answer = 0.0
      end if
   elseif (Direction=='E') then
      if (Primary=='P') then
         NodeDist = (XNodeWidths(I)+XNodeWidths(I+1))/2.D0
         Answer = -TempGamma1X(I,J,K)/(NodeDist*XNodeWidths(I))
      elseif (Primary=='T') then
         NodeDist = (XNodeWidths(I)+XNodeWidths(I+1))/2.D0
         Answer = -XInterfEffCond(I,J,K)/(NodeDist*XNodeWidths(I))
      elseif (Primary=='X') then
         Answer = 0.0
      end if
   elseif (Direction=='T') then
      if (Primary=='P') then
         NodeDist = (ZNodeWidths(K-1)+ZNodeWidths(K))/2.D0
         Answer = -TempGamma1Z(I,J,K-1)/(NodeDist*ZNodeWidths(K))
      elseif (Primary=='T') then
         NodeDist = (ZNodeWidths(K-1)+ZNodeWidths(K))/2.D0
         Answer = -ZInterfEffCond(I,J,K-1)/(NodeDist*ZNodeWidths(K))
      elseif (Primary=='X') then
         Answer = 0.0
      end if
   elseif (Direction=='B') then
      if (Primary=='P') then
         NodeDist = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
         Answer = -TempGamma1Z(I,J,K)/(NodeDist*ZNodeWidths(K))
      elseif (Primary=='T') then
         NodeDist = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
         Answer = -ZInterfEffCond(I,J,K)/(NodeDist*ZNodeWidths(K))
      elseif (Primary=='X') then
         Answer = 0.0
      end if
   end if
end if
Answer = Answer*3.2D4
end
