subroutine ReadInFile2Array
! Purpose: To read user defined parameters from the file InFile2
!          into arrays.

use commonvars, only: StressNodeTypes,NumNodesX,NumNodesY,NumNodesZ, &
   StressBoundVals,YoungModulus,Prec,LogFile,PoissonRatio,SolveStress
implicit none
character :: FreeOrConst
integer :: I,J,K
real(selected_real_kind(Prec)) :: ConstYoungModulus,ConstPoissonRatio

read(10,*)
read(10,*) SolveStress

if (SolveStress==1) then
!----
read(10,*)
read(10,*)

! -------------------------------------------------------------------
!                       Stress Node Types
! -------------------------------------------------------------------
do J=1,NumNodesY
   do K=1,NumNodesZ
      read(10,*) (StressNodeTypes(I,J,K),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      read(10,*)
   end if
end do

50 format(2000E15.8) ! Stress Boundary Values

write(8,*) 
write(8,*) '--- Stress Node Types ---'
write(8,*) '([S]tress Boundary, [R]oller Boundary,'
write(8,*) ' [P]inned Boundary, [I]nterior)'
write(8,*)
do J=1,NumNodesY
   do K=1,NumNodesZ
      write(8,*) (StressNodeTypes(I,J,K),I=1,NumNodesX)
   end do
   write(8,*)
end do
! -------------------------------------------------------------------
!                       Stress Boundary Values
! -------------------------------------------------------------------
read(10,*)
do J=1,NumNodesY
   do K=1,NumNodesZ
      read(10,*) (StressBoundVals(I,J,K),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      read(10,*)
   end if
end do

write(8,*) '--- Stress Boundary Values (0=not a boundary) ---'
write(8,*)
do J=1,NumNodesY
   do K=1,NumNodesZ
      write(8,50) (real(StressBoundVals(I,J,K)),I=1,NumNodesX)
   end do
   write(8,*)
end do

!--------------------------------------------------------------------
!                       Young's Moduli (Pa)
!--------------------------------------------------------------------
read(10,*)
read(10,*) FreeOrConst
if (FreeOrConst=='C') then
   read(10,*) ConstYoungModulus
   do I=1,NumNodesX
      do J=1,NumNodesY
         do K=1,NumNodesZ
            YoungModulus(I,J,K) = ConstYoungModulus
         end do
      end do
   end do
elseif (FreeOrConst=='F') then
   do J=1,NumNodesY
      do K=1,NumNodesZ
         read(10,*) (YoungModulus(I,J,K),I=1,NumNodesX)
      end do
      if (J<NumNodesY) then
         read(10,*)
      end if
   end do
else
   write(LogFile,*) 'Invalid Value of FreeOrConst: ',FreeOrConst
   STOP
end if

10 format(2000E10.3) ! Young's moduli, Poisson's ratios

write(8,*)
write(8,*) '--- Young Modulus (Pa) ---'
do J=1,NumNodesY
   write(8,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(8,10) (real(YoungModulus(I,J,K)),I=1,NumNodesX)
   end do
end do
!--------------------------------------------------------------------
!                       Poisson's Ratio (unitless)
!--------------------------------------------------------------------
read(10,*)
read(10,*) FreeOrConst
if (FreeOrConst=='C') then
   read(10,*) ConstPoissonRatio
   do I=1,NumNodesX
      do J=1,NumNodesY
         do K=1,NumNodesZ
            PoissonRatio(I,J,K) = ConstPoissonRatio
         end do
      end do
   end do
elseif (FreeOrConst=='F') then
   do J=1,NumNodesY
      do K=1,NumNodesZ
         read(10,*) (PoissonRatio(I,J,K),I=1,NumNodesX)
      end do
      if (J<NumNodesY) then
         read(10,*)
      end if
   end do
else
   write(LogFile,*) 'Invalid Value of FreeOrConst: ',FreeOrConst
   STOP
end if

write(8,*)
write(8,*) '--- Poisson Ratio ---'
do J=1,NumNodesY
   write(8,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(8,10) (real(PoissonRatio(I,J,K)),I=1,NumNodesX)
   end do
end do
!----
end if
end
